# See the README.md file in this directory for more information.

from typing import Any, Protocol

# As defined https://docs.python.org/3/library/xml.dom.html#domimplementation-objects
class DOMImplementation(Protocol):
    def hasFeature(self, __feature: str, __version: str | None) -> bool: ...
    def createDocument(self, __namespaceUri: str, __qualifiedName: str, __doctype: Any | None) -> Any: ...
    def createDocumentType(self, __qualifiedName: str, __publicId: str, __systemId: str) -> Any: ...



from collections.abc import Callable, Iterable
from typing import Any

from django.core.exceptions import ImproperlyConfigured

class InvalidCacheBackendError(ImproperlyConfigured): ...
class CacheKeyWarning(RuntimeWarning): ...
class InvalidCacheKey(ValueError): ...

DEFAULT_TIMEOUT: Any
MEMCACHE_MAX_KEY_LENGTH: int

def default_key_func(key: Any, key_prefix: str, version: Any) -> str: ...
def get_key_func(key_func: Callable | str | None) -> Callable: ...

class BaseCache:
    _missing_key: object
    default_timeout: float | None
    _max_entries: int
    _cull_frequency: int
    key_prefix: str
    version: int
    key_func: Callable
    def __init__(self, params: dict[str, Any]) -> None: ...
    def get_backend_timeout(self, timeout: float | None = ...) -> float | None: ...
    def make_key(self, key: Any, version: int | None = ...) -> str: ...
    def validate_key(self, key: Any) -> None: ...
    def make_and_validate_key(self, key: Any, version: int | None = ...) -> str: ...
    def add(self, key: Any, value: Any, timeout: float | None = ..., version: int | None = ...) -> bool: ...
    async def aadd(self, key: Any, value: Any, timeout: float | None = ..., version: int | None = ...) -> bool: ...
    def get(self, key: Any, default: Any | None = ..., version: int | None = ...) -> Any: ...
    async def aget(self, key: Any, default: Any | None = ..., version: int | None = ...) -> Any: ...
    def set(self, key: Any, value: Any, timeout: float | None = ..., version: int | None = ...) -> None: ...
    async def aset(self, key: Any, value: Any, timeout: float | None = ..., version: int | None = ...) -> None: ...
    def touch(self, key: Any, timeout: float | None = ..., version: int | None = ...) -> bool: ...
    async def atouch(self, key: Any, timeout: float | None = ..., version: int | None = ...) -> bool: ...
    def delete(self, key: Any, version: int | None = ...) -> bool: ...
    async def adelete(self, key: Any, version: int | None = ...) -> bool: ...
    def get_many(self, keys: Iterable[Any], version: int | None = ...) -> dict[Any, Any]: ...
    async def aget_many(self, keys: Iterable[Any], version: int | None = ...) -> dict[Any, Any]: ...
    def get_or_set(
        self, key: Any, default: Any | None, timeout: float | None = ..., version: int | None = ...
    ) -> Any | None: ...
    async def aget_or_set(
        self, key: Any, default: Any | None, timeout: float | None = ..., version: int | None = ...
    ) -> Any | None: ...
    def has_key(self, key: Any, version: int | None = ...) -> bool: ...
    async def ahas_key(self, key: Any, version: int | None = ...) -> bool: ...
    def incr(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    async def aincr(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    def decr(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    async def adecr(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    def __contains__(self, key: Any) -> bool: ...
    def set_many(self, data: dict[Any, Any], timeout: float | None = ..., version: int | None = ...) -> list[Any]: ...
    async def aset_many(
        self, data: dict[Any, Any], timeout: float | None = ..., version: int | None = ...
    ) -> list[Any]: ...
    def delete_many(self, keys: Iterable[Any], version: int | None = ...) -> None: ...
    async def adelete_many(self, keys: Iterable[Any], version: int | None = ...) -> None: ...
    def clear(self) -> None: ...
    async def aclear(self) -> None: ...
    def incr_version(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    async def aincr_version(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    def decr_version(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    async def adecr_version(self, key: Any, delta: int = ..., version: int | None = ...) -> int: ...
    def close(self, **kwargs: Any) -> None: ...
    async def aclose(self, **kwargs: Any) -> None: ...



import datetime
from collections.abc import Generator, Iterator, Mapping, Sequence
from contextlib import contextmanager
from decimal import Decimal
from logging import Logger
from types import TracebackType
from typing import Any, Protocol, overload
from uuid import UUID

from _typeshed import Self
from typing_extensions import Literal, TypeAlias

logger: Logger

# Protocol matching psycopg2.sql.Composable, to avoid depending psycopg2
class _Composable(Protocol):
    def as_string(self, context: Any) -> str: ...
    def __add__(self, other: _Composable) -> _Composable: ...
    def __mul__(self, n: int) -> _Composable: ...

_ExecuteQuery: TypeAlias = str | _Composable

# Python types that can be adapted to SQL.
_SQLType: TypeAlias = (
    None
    | bool
    | int
    | float
    | Decimal
    | str
    | bytes
    | datetime.date
    | datetime.datetime
    | UUID
    | tuple[Any, ...]
    | list[Any]
)
_ExecuteParameters: TypeAlias = Sequence[_SQLType] | Mapping[str, _SQLType] | None

class CursorWrapper:
    cursor: Any
    db: Any
    def __init__(self, cursor: Any, db: Any) -> None: ...
    WRAP_ERROR_ATTRS: Any
    def __getattr__(self, attr: str) -> Any: ...
    def __iter__(self) -> Iterator[tuple[Any, ...]]: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def callproc(
        self, procname: str, params: Sequence[Any] | None = ..., kparams: dict[str, int] | None = ...
    ) -> Any: ...
    def execute(self, sql: _ExecuteQuery, params: _ExecuteParameters = ...) -> Any: ...
    def executemany(self, sql: _ExecuteQuery, param_list: Sequence[_ExecuteParameters]) -> Any: ...

class CursorDebugWrapper(CursorWrapper):
    cursor: Any
    db: Any
    @contextmanager
    def debug_sql(
        self,
        sql: str | None = ...,
        params: _ExecuteParameters | Sequence[_ExecuteParameters] | None = ...,
        use_last_executed_query: bool = ...,
        many: bool = ...,
    ) -> Generator[None, None, None]: ...

@overload
def typecast_date(s: None | Literal[""]) -> None: ...  # type: ignore
@overload
def typecast_date(s: str) -> datetime.date: ...
@overload
def typecast_time(s: None | Literal[""]) -> None: ...  # type: ignore
@overload
def typecast_time(s: str) -> datetime.time: ...
@overload
def typecast_timestamp(s: None | Literal[""]) -> None: ...  # type: ignore
@overload
def typecast_timestamp(s: str) -> datetime.datetime: ...
def split_identifier(identifier: str) -> tuple[str, str]: ...
def truncate_name(identifier: str, length: int | None = ..., hash_len: int = ...) -> str: ...
def format_number(value: Decimal | None, max_digits: int | None, decimal_places: int | None) -> str | None: ...
def strip_quotes(table_name: str) -> str: ...



import sys
from _typeshed import SupportsWrite
from collections.abc import Iterable, Iterator
from typing import Any
from typing_extensions import Final, Literal, TypeAlias

__version__: Final[str]

QUOTE_ALL: Literal[1]
QUOTE_MINIMAL: Literal[0]
QUOTE_NONE: Literal[3]
QUOTE_NONNUMERIC: Literal[2]
if sys.version_info >= (3, 12):
    QUOTE_STRINGS: Literal[4]
    QUOTE_NOTNULL: Literal[5]

# Ideally this would be `QUOTE_ALL | QUOTE_MINIMAL | QUOTE_NONE | QUOTE_NONNUMERIC`
# However, using literals in situations like these can cause false-positives (see #7258)
_QuotingType: TypeAlias = int

class Error(Exception): ...

class Dialect:
    delimiter: str
    quotechar: str | None
    escapechar: str | None
    doublequote: bool
    skipinitialspace: bool
    lineterminator: str
    quoting: _QuotingType
    strict: bool
    def __init__(self) -> None: ...

_DialectLike: TypeAlias = str | Dialect | type[Dialect]

class _reader(Iterator[list[str]]):
    @property
    def dialect(self) -> Dialect: ...
    line_num: int
    def __next__(self) -> list[str]: ...

class _writer:
    @property
    def dialect(self) -> Dialect: ...
    def writerow(self, row: Iterable[Any]) -> Any: ...
    def writerows(self, rows: Iterable[Iterable[Any]]) -> None: ...

def writer(
    csvfile: SupportsWrite[str],
    dialect: _DialectLike = "excel",
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> _writer: ...
def reader(
    csvfile: Iterable[str],
    dialect: _DialectLike = "excel",
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> _reader: ...
def register_dialect(
    name: str,
    dialect: type[Dialect] = ...,
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> None: ...
def unregister_dialect(name: str) -> None: ...
def get_dialect(name: str) -> Dialect: ...
def list_dialects() -> list[str]: ...
def field_size_limit(new_limit: int = ...) -> int: ...



from pydantic import AliasGenerator, BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class PydanticBaseDjango(BaseModel):
    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel),
        populate_by_name=True,
        extra="forbid",
    )


__all__ = ["PydanticBaseDjango"]



import sys
import unittest.case
from _typeshed import OptExcInfo
from collections.abc import Callable
from typing import Any, TextIO, TypeVar
from typing_extensions import TypeAlias

_F = TypeVar("_F", bound=Callable[..., Any])
_DurationsType: TypeAlias = list[tuple[str, float]]

STDOUT_LINE: str
STDERR_LINE: str

# undocumented
def failfast(method: _F) -> _F: ...

class TestResult:
    errors: list[tuple[unittest.case.TestCase, str]]
    failures: list[tuple[unittest.case.TestCase, str]]
    skipped: list[tuple[unittest.case.TestCase, str]]
    expectedFailures: list[tuple[unittest.case.TestCase, str]]
    unexpectedSuccesses: list[unittest.case.TestCase]
    shouldStop: bool
    testsRun: int
    buffer: bool
    failfast: bool
    tb_locals: bool
    if sys.version_info >= (3, 12):
        collectedDurations: _DurationsType
    def __init__(self, stream: TextIO | None = None, descriptions: bool | None = None, verbosity: int | None = None) -> None: ...
    def printErrors(self) -> None: ...
    def wasSuccessful(self) -> bool: ...
    def stop(self) -> None: ...
    def startTest(self, test: unittest.case.TestCase) -> None: ...
    def stopTest(self, test: unittest.case.TestCase) -> None: ...
    def startTestRun(self) -> None: ...
    def stopTestRun(self) -> None: ...
    def addError(self, test: unittest.case.TestCase, err: OptExcInfo) -> None: ...
    def addFailure(self, test: unittest.case.TestCase, err: OptExcInfo) -> None: ...
    def addSuccess(self, test: unittest.case.TestCase) -> None: ...
    def addSkip(self, test: unittest.case.TestCase, reason: str) -> None: ...
    def addExpectedFailure(self, test: unittest.case.TestCase, err: OptExcInfo) -> None: ...
    def addUnexpectedSuccess(self, test: unittest.case.TestCase) -> None: ...
    def addSubTest(self, test: unittest.case.TestCase, subtest: unittest.case.TestCase, err: OptExcInfo | None) -> None: ...
    if sys.version_info >= (3, 12):
        def addDuration(self, test: unittest.case.TestCase, elapsed: float) -> None: ...



from ctypes import Structure
from logging import Logger
from typing import Any

logger: Logger

def load_geos() -> Any: ...

NOTICEFUNC: Any

def notice_h(fmt: Any, lst: Any) -> None: ...

ERRORFUNC: Any

def error_h(fmt: Any, lst: Any) -> None: ...

class GEOSGeom_t(Structure): ...
class GEOSPrepGeom_t(Structure): ...
class GEOSCoordSeq_t(Structure): ...
class GEOSContextHandle_t(Structure): ...

GEOM_PTR: Any
PREPGEOM_PTR: Any
CS_PTR: Any
CONTEXT_PTR: Any
lgeos: Any

class GEOSFuncFactory:
    argtypes: Any
    restype: Any
    errcheck: Any
    func_name: Any
    def __init__(
        self, func_name: Any, *, restype: Any | None = ..., errcheck: Any | None = ..., argtypes: Any | None = ...
    ) -> None: ...
    def __call__(self, *args: Any) -> Any: ...
    @property
    def func(self) -> Any: ...

def geos_version() -> bytes: ...
def geos_version_tuple() -> tuple[int, ...]: ...



from typing import Generic, TypeVar

import factory

T = TypeVar("T")


class BaseMetaFactory(Generic[T], factory.base.FactoryMetaClass):
    def __call__(cls, *args, **kwargs) -> T:
        return super().__call__(*args, **kwargs)



import sys
from collections.abc import Iterator
from typing import Any, Protocol, TypeVar, overload

_T = TypeVar("_T")

class PackageMetadata(Protocol):
    def __len__(self) -> int: ...
    def __contains__(self, item: str) -> bool: ...
    def __getitem__(self, key: str) -> str: ...
    def __iter__(self) -> Iterator[str]: ...
    @property
    def json(self) -> dict[str, str | list[str]]: ...
    @overload
    def get_all(self, name: str, failobj: None = None) -> list[Any] | None: ...
    @overload
    def get_all(self, name: str, failobj: _T) -> list[Any] | _T: ...
    if sys.version_info >= (3, 12):
        @overload
        def get(self, name: str, failobj: None = None) -> str | None: ...
        @overload
        def get(self, name: str, failobj: _T) -> _T | str: ...

if sys.version_info >= (3, 12):
    class SimplePath(Protocol[_T]):
        def joinpath(self) -> _T: ...
        @property
        def parent(self) -> _T: ...
        def read_text(self) -> str: ...
        def __truediv__(self, other: _T | str) -> _T: ...

else:
    class SimplePath(Protocol):
        def joinpath(self) -> SimplePath: ...
        def parent(self) -> SimplePath: ...
        def read_text(self) -> str: ...
        # There was a bug in `SimplePath` definition in cpython, see #8451
        #  Strictly speaking `__div__` was defined in 3.10, not __truediv__,
        # but it should have always been `__truediv__`.
        def __truediv__(self) -> SimplePath: ...



import io
import sys
from _typeshed import (
    BytesPath,
    GenericPath,
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
    ReadableBuffer,
    StrPath,
    WriteableBuffer,
)
from collections.abc import Iterable, Iterator
from types import TracebackType
from typing import IO, Any, AnyStr, Generic, Literal, overload
from typing_extensions import Self

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "NamedTemporaryFile",
    "TemporaryFile",
    "SpooledTemporaryFile",
    "TemporaryDirectory",
    "mkstemp",
    "mkdtemp",
    "mktemp",
    "TMP_MAX",
    "gettempprefix",
    "tempdir",
    "gettempdir",
    "gettempprefixb",
    "gettempdirb",
]

# global variables
TMP_MAX: int
tempdir: str | None
template: str

if sys.version_info >= (3, 12):
    @overload
    def NamedTemporaryFile(
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
        delete_on_close: bool = True,
    ) -> _TemporaryFileWrapper[str]: ...
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryMode = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
        delete_on_close: bool = True,
    ) -> _TemporaryFileWrapper[bytes]: ...
    @overload
    def NamedTemporaryFile(
        mode: str = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
        delete_on_close: bool = True,
    ) -> _TemporaryFileWrapper[Any]: ...

else:
    @overload
    def NamedTemporaryFile(
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
    ) -> _TemporaryFileWrapper[str]: ...
    @overload
    def NamedTemporaryFile(
        mode: OpenBinaryMode = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
    ) -> _TemporaryFileWrapper[bytes]: ...
    @overload
    def NamedTemporaryFile(
        mode: str = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        delete: bool = True,
        *,
        errors: str | None = None,
    ) -> _TemporaryFileWrapper[Any]: ...

if sys.platform == "win32":
    TemporaryFile = NamedTemporaryFile
else:
    # See the comments for builtins.open() for an explanation of the overloads.
    @overload
    def TemporaryFile(
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> io.TextIOWrapper: ...
    @overload
    def TemporaryFile(
        mode: OpenBinaryMode,
        buffering: Literal[0],
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> io.FileIO: ...
    @overload
    def TemporaryFile(
        *,
        buffering: Literal[0],
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        errors: str | None = None,
    ) -> io.FileIO: ...
    @overload
    def TemporaryFile(
        mode: OpenBinaryModeWriting,
        buffering: Literal[-1, 1] = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> io.BufferedWriter: ...
    @overload
    def TemporaryFile(
        mode: OpenBinaryModeReading,
        buffering: Literal[-1, 1] = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> io.BufferedReader: ...
    @overload
    def TemporaryFile(
        mode: OpenBinaryModeUpdating = "w+b",
        buffering: Literal[-1, 1] = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> io.BufferedRandom: ...
    @overload
    def TemporaryFile(
        mode: str = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: AnyStr | None = None,
        prefix: AnyStr | None = None,
        dir: GenericPath[AnyStr] | None = None,
        *,
        errors: str | None = None,
    ) -> IO[Any]: ...

class _TemporaryFileWrapper(IO[AnyStr]):
    file: IO[AnyStr]  # io.TextIOWrapper, io.BufferedReader or io.BufferedWriter
    name: str
    delete: bool
    if sys.version_info >= (3, 12):
        def __init__(self, file: IO[AnyStr], name: str, delete: bool = True, delete_on_close: bool = True) -> None: ...
    else:
        def __init__(self, file: IO[AnyStr], name: str, delete: bool = True) -> None: ...

    def __enter__(self) -> Self: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def close(self) -> None: ...
    # These methods don't exist directly on this object, but
    # are delegated to the underlying IO object through __getattr__.
    # We need to add them here so that this class is concrete.
    def __iter__(self) -> Iterator[AnyStr]: ...
    # FIXME: __next__ doesn't actually exist on this class and should be removed:
    #        see also https://github.com/python/typeshed/pull/5456#discussion_r633068648
    # >>> import tempfile
    # >>> ntf=tempfile.NamedTemporaryFile()
    # >>> next(ntf)
    # Traceback (most recent call last):
    #   File "<stdin>", line 1, in <module>
    # TypeError: '_TemporaryFileWrapper' object is not an iterator
    def __next__(self) -> AnyStr: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def read(self, n: int = ...) -> AnyStr: ...
    def readable(self) -> bool: ...
    def readline(self, limit: int = ...) -> AnyStr: ...
    def readlines(self, hint: int = ...) -> list[AnyStr]: ...
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, size: int | None = ...) -> int: ...
    def writable(self) -> bool: ...
    @overload
    def write(self: _TemporaryFileWrapper[str], s: str) -> int: ...
    @overload
    def write(self: _TemporaryFileWrapper[bytes], s: ReadableBuffer) -> int: ...
    @overload
    def write(self, s: AnyStr) -> int: ...
    @overload
    def writelines(self: _TemporaryFileWrapper[str], lines: Iterable[str]) -> None: ...
    @overload
    def writelines(self: _TemporaryFileWrapper[bytes], lines: Iterable[ReadableBuffer]) -> None: ...
    @overload
    def writelines(self, lines: Iterable[AnyStr]) -> None: ...

if sys.version_info >= (3, 11):
    _SpooledTemporaryFileBase = io.IOBase
else:
    _SpooledTemporaryFileBase = object

# It does not actually derive from IO[AnyStr], but it does mostly behave
# like one.
class SpooledTemporaryFile(IO[AnyStr], _SpooledTemporaryFileBase):
    _file: IO[AnyStr]
    @property
    def encoding(self) -> str: ...  # undocumented
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...  # undocumented
    # bytes needs to go first, as default mode is to open as bytes
    @overload
    def __init__(
        self: SpooledTemporaryFile[bytes],
        max_size: int = 0,
        mode: OpenBinaryMode = "w+b",
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: str | None = None,
        *,
        errors: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: SpooledTemporaryFile[str],
        max_size: int,
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: str | None = None,
        *,
        errors: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: SpooledTemporaryFile[str],
        max_size: int = 0,
        *,
        mode: OpenTextMode,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: str | None = None,
        errors: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        max_size: int,
        mode: str,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: str | None = None,
        *,
        errors: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        max_size: int = 0,
        *,
        mode: str,
        buffering: int = -1,
        encoding: str | None = None,
        newline: str | None = None,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: str | None = None,
        errors: str | None = None,
    ) -> None: ...
    @property
    def errors(self) -> str | None: ...
    def rollover(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    # These methods are copied from the abstract methods of IO, because
    # SpooledTemporaryFile implements IO.
    # See also https://github.com/python/typeshed/pull/2452#issuecomment-420657918.
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    if sys.version_info >= (3, 11):
        # These three work only if the SpooledTemporaryFile is opened in binary mode,
        # because the underlying object in text mode does not have these methods.
        def read1(self, size: int = ..., /) -> AnyStr: ...
        def readinto(self, b: WriteableBuffer) -> int: ...
        def readinto1(self, b: WriteableBuffer) -> int: ...
        def detach(self) -> io.RawIOBase: ...

    def read(self, n: int = ..., /) -> AnyStr: ...
    def readline(self, limit: int | None = ..., /) -> AnyStr: ...  # type: ignore[override]
    def readlines(self, hint: int = ..., /) -> list[AnyStr]: ...  # type: ignore[override]
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def tell(self) -> int: ...
    def truncate(self, size: int | None = None) -> None: ...  # type: ignore[override]
    @overload
    def write(self: SpooledTemporaryFile[str], s: str) -> int: ...
    @overload
    def write(self: SpooledTemporaryFile[bytes], s: ReadableBuffer) -> int: ...
    @overload
    def write(self, s: AnyStr) -> int: ...
    @overload
    def writelines(self: SpooledTemporaryFile[str], iterable: Iterable[str]) -> None: ...
    @overload
    def writelines(self: SpooledTemporaryFile[bytes], iterable: Iterable[ReadableBuffer]) -> None: ...
    @overload
    def writelines(self, iterable: Iterable[AnyStr]) -> None: ...
    def __iter__(self) -> Iterator[AnyStr]: ...  # type: ignore[override]
    # These exist at runtime only on 3.11+.
    def readable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def writable(self) -> bool: ...
    def __next__(self) -> AnyStr: ...  # type: ignore[override]
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class TemporaryDirectory(Generic[AnyStr]):
    name: AnyStr
    if sys.version_info >= (3, 12):
        @overload
        def __init__(
            self: TemporaryDirectory[str],
            suffix: str | None = None,
            prefix: str | None = None,
            dir: StrPath | None = None,
            ignore_cleanup_errors: bool = False,
            *,
            delete: bool = True,
        ) -> None: ...
        @overload
        def __init__(
            self: TemporaryDirectory[bytes],
            suffix: bytes | None = None,
            prefix: bytes | None = None,
            dir: BytesPath | None = None,
            ignore_cleanup_errors: bool = False,
            *,
            delete: bool = True,
        ) -> None: ...
    elif sys.version_info >= (3, 10):
        @overload
        def __init__(
            self: TemporaryDirectory[str],
            suffix: str | None = None,
            prefix: str | None = None,
            dir: StrPath | None = None,
            ignore_cleanup_errors: bool = False,
        ) -> None: ...
        @overload
        def __init__(
            self: TemporaryDirectory[bytes],
            suffix: bytes | None = None,
            prefix: bytes | None = None,
            dir: BytesPath | None = None,
            ignore_cleanup_errors: bool = False,
        ) -> None: ...
    else:
        @overload
        def __init__(
            self: TemporaryDirectory[str], suffix: str | None = None, prefix: str | None = None, dir: StrPath | None = None
        ) -> None: ...
        @overload
        def __init__(
            self: TemporaryDirectory[bytes],
            suffix: bytes | None = None,
            prefix: bytes | None = None,
            dir: BytesPath | None = None,
        ) -> None: ...

    def cleanup(self) -> None: ...
    def __enter__(self) -> AnyStr: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

# The overloads overlap, but they should still work fine.
@overload
def mkstemp(  # type: ignore[overload-overlap]
    suffix: str | None = None, prefix: str | None = None, dir: StrPath | None = None, text: bool = False
) -> tuple[int, str]: ...
@overload
def mkstemp(
    suffix: bytes | None = None, prefix: bytes | None = None, dir: BytesPath | None = None, text: bool = False
) -> tuple[int, bytes]: ...

# The overloads overlap, but they should still work fine.
@overload
def mkdtemp(suffix: str | None = None, prefix: str | None = None, dir: StrPath | None = None) -> str: ...  # type: ignore[overload-overlap]
@overload
def mkdtemp(suffix: bytes | None = None, prefix: bytes | None = None, dir: BytesPath | None = None) -> bytes: ...
def mktemp(suffix: str = "", prefix: str = "tmp", dir: StrPath | None = None) -> str: ...
def gettempdirb() -> bytes: ...
def gettempprefixb() -> bytes: ...
def gettempdir() -> str: ...
def gettempprefix() -> str: ...



from collections.abc import Callable, Mapping, Sequence
from typing import Any

color_names: Sequence
foreground: Mapping[str, str]
background: Mapping[str, str]
RESET: str
opt_dict: Mapping[str, str]

def colorize(text: str | None = ..., opts: Sequence[str] = ..., **kwargs: Any) -> str: ...
def make_style(opts: tuple = ..., **kwargs: Any) -> Callable: ...

NOCOLOR_PALETTE: str
DARK_PALETTE: str
LIGHT_PALETTE: str
PALETTES: Any
DEFAULT_PALETTE: str

def parse_color_setting(config_string: str) -> dict[str, dict[str, tuple[str, ...] | str]] | None: ...



from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.utils.deprecation import MiddlewareMixin

class RedirectFallbackMiddleware(MiddlewareMixin):
    response_gone_class: Any
    response_redirect_class: Any
    def process_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse: ...



import logging
import sys
import unittest.result
from _typeshed import SupportsDunderGE, SupportsDunderGT, SupportsDunderLE, SupportsDunderLT, SupportsRSub, SupportsSub
from collections.abc import Callable, Container, Iterable, Mapping, Sequence, Set as AbstractSet
from contextlib import AbstractContextManager
from re import Pattern
from types import TracebackType
from typing import Any, AnyStr, ClassVar, Generic, NamedTuple, NoReturn, Protocol, SupportsAbs, SupportsRound, TypeVar, overload
from typing_extensions import ParamSpec, Self, TypeAlias
from warnings import WarningMessage

if sys.version_info >= (3, 9):
    from types import GenericAlias

if sys.version_info >= (3, 10):
    from types import UnionType

_T = TypeVar("_T")
_S = TypeVar("_S", bound=SupportsSub[Any, Any])
_E = TypeVar("_E", bound=BaseException)
_FT = TypeVar("_FT", bound=Callable[..., Any])
_P = ParamSpec("_P")

DIFF_OMITTED: str

class _BaseTestCaseContext:
    def __init__(self, test_case: TestCase) -> None: ...

if sys.version_info >= (3, 9):
    from unittest._log import _AssertLogsContext, _LoggingWatcher
else:
    # Unused dummy for _AssertLogsContext. Starting with Python 3.10,
    # this is generic over the logging watcher, but in lower versions
    # the watcher is hard-coded.
    _L = TypeVar("_L")

    class _LoggingWatcher(NamedTuple):
        records: list[logging.LogRecord]
        output: list[str]

    class _AssertLogsContext(_BaseTestCaseContext, Generic[_L]):
        LOGGING_FORMAT: ClassVar[str]
        test_case: TestCase
        logger_name: str
        level: int
        msg: None
        def __init__(self, test_case: TestCase, logger_name: str, level: int) -> None: ...
        def __enter__(self) -> _LoggingWatcher: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, tb: TracebackType | None
        ) -> bool | None: ...

if sys.version_info >= (3, 8):
    def addModuleCleanup(__function: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    def doModuleCleanups() -> None: ...

if sys.version_info >= (3, 11):
    def enterModuleContext(cm: AbstractContextManager[_T]) -> _T: ...

def expectedFailure(test_item: _FT) -> _FT: ...
def skip(reason: str) -> Callable[[_FT], _FT]: ...
def skipIf(condition: object, reason: str) -> Callable[[_FT], _FT]: ...
def skipUnless(condition: object, reason: str) -> Callable[[_FT], _FT]: ...

class SkipTest(Exception):
    def __init__(self, reason: str) -> None: ...

class _SupportsAbsAndDunderGE(SupportsDunderGE[Any], SupportsAbs[Any], Protocol): ...

# Keep this alias in sync with builtins._ClassInfo
# We can't import it from builtins or pytype crashes,
# due to the fact that pytype uses a custom builtins stub rather than typeshed's builtins stub
if sys.version_info >= (3, 10):
    _ClassInfo: TypeAlias = type | UnionType | tuple[_ClassInfo, ...]
else:
    _ClassInfo: TypeAlias = type | tuple[_ClassInfo, ...]

class TestCase:
    failureException: type[BaseException]
    longMessage: bool
    maxDiff: int | None
    # undocumented
    _testMethodName: str
    # undocumented
    _testMethodDoc: str
    def __init__(self, methodName: str = "runTest") -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    def run(self, result: unittest.result.TestResult | None = None) -> unittest.result.TestResult | None: ...
    def __call__(self, result: unittest.result.TestResult | None = ...) -> unittest.result.TestResult | None: ...
    def skipTest(self, reason: Any) -> NoReturn: ...
    def subTest(self, msg: Any = ..., **params: Any) -> AbstractContextManager[None]: ...
    def debug(self) -> None: ...
    if sys.version_info < (3, 11):
        def _addSkip(self, result: unittest.result.TestResult, test_case: TestCase, reason: str) -> None: ...

    def assertEqual(self, first: Any, second: Any, msg: Any = None) -> None: ...
    def assertNotEqual(self, first: Any, second: Any, msg: Any = None) -> None: ...
    def assertTrue(self, expr: Any, msg: Any = None) -> None: ...
    def assertFalse(self, expr: Any, msg: Any = None) -> None: ...
    def assertIs(self, expr1: object, expr2: object, msg: Any = None) -> None: ...
    def assertIsNot(self, expr1: object, expr2: object, msg: Any = None) -> None: ...
    def assertIsNone(self, obj: object, msg: Any = None) -> None: ...
    def assertIsNotNone(self, obj: object, msg: Any = None) -> None: ...
    def assertIn(self, member: Any, container: Iterable[Any] | Container[Any], msg: Any = None) -> None: ...
    def assertNotIn(self, member: Any, container: Iterable[Any] | Container[Any], msg: Any = None) -> None: ...
    def assertIsInstance(self, obj: object, cls: _ClassInfo, msg: Any = None) -> None: ...
    def assertNotIsInstance(self, obj: object, cls: _ClassInfo, msg: Any = None) -> None: ...
    @overload
    def assertGreater(self, a: SupportsDunderGT[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertGreater(self, a: _T, b: SupportsDunderLT[_T], msg: Any = None) -> None: ...
    @overload
    def assertGreaterEqual(self, a: SupportsDunderGE[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertGreaterEqual(self, a: _T, b: SupportsDunderLE[_T], msg: Any = None) -> None: ...
    @overload
    def assertLess(self, a: SupportsDunderLT[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertLess(self, a: _T, b: SupportsDunderGT[_T], msg: Any = None) -> None: ...
    @overload
    def assertLessEqual(self, a: SupportsDunderLE[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertLessEqual(self, a: _T, b: SupportsDunderGE[_T], msg: Any = None) -> None: ...
    # `assertRaises`, `assertRaisesRegex`, and `assertRaisesRegexp`
    # are not using `ParamSpec` intentionally,
    # because they might be used with explicitly wrong arg types to raise some error in tests.
    @overload
    def assertRaises(
        self,
        expected_exception: type[BaseException] | tuple[type[BaseException], ...],
        callable: Callable[..., object],
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    @overload
    def assertRaises(
        self, expected_exception: type[_E] | tuple[type[_E], ...], *, msg: Any = ...
    ) -> _AssertRaisesContext[_E]: ...
    @overload
    def assertRaisesRegex(
        self,
        expected_exception: type[BaseException] | tuple[type[BaseException], ...],
        expected_regex: str | Pattern[str],
        callable: Callable[..., object],
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    @overload
    def assertRaisesRegex(
        self, expected_exception: type[_E] | tuple[type[_E], ...], expected_regex: str | Pattern[str], *, msg: Any = ...
    ) -> _AssertRaisesContext[_E]: ...
    @overload
    def assertWarns(
        self,
        expected_warning: type[Warning] | tuple[type[Warning], ...],
        callable: Callable[_P, object],
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> None: ...
    @overload
    def assertWarns(
        self, expected_warning: type[Warning] | tuple[type[Warning], ...], *, msg: Any = ...
    ) -> _AssertWarnsContext: ...
    @overload
    def assertWarnsRegex(
        self,
        expected_warning: type[Warning] | tuple[type[Warning], ...],
        expected_regex: str | Pattern[str],
        callable: Callable[_P, object],
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> None: ...
    @overload
    def assertWarnsRegex(
        self, expected_warning: type[Warning] | tuple[type[Warning], ...], expected_regex: str | Pattern[str], *, msg: Any = ...
    ) -> _AssertWarnsContext: ...
    def assertLogs(
        self, logger: str | logging.Logger | None = None, level: int | str | None = None
    ) -> _AssertLogsContext[_LoggingWatcher]: ...
    if sys.version_info >= (3, 10):
        def assertNoLogs(
            self, logger: str | logging.Logger | None = None, level: int | str | None = None
        ) -> _AssertLogsContext[None]: ...

    @overload
    def assertAlmostEqual(self, first: _S, second: _S, places: None, msg: Any, delta: _SupportsAbsAndDunderGE) -> None: ...
    @overload
    def assertAlmostEqual(
        self, first: _S, second: _S, places: None = None, msg: Any = None, *, delta: _SupportsAbsAndDunderGE
    ) -> None: ...
    @overload
    def assertAlmostEqual(
        self,
        first: SupportsSub[_T, SupportsAbs[SupportsRound[object]]],
        second: _T,
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    @overload
    def assertAlmostEqual(
        self,
        first: _T,
        second: SupportsRSub[_T, SupportsAbs[SupportsRound[object]]],
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    @overload
    def assertNotAlmostEqual(self, first: _S, second: _S, places: None, msg: Any, delta: _SupportsAbsAndDunderGE) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self, first: _S, second: _S, places: None = None, msg: Any = None, *, delta: _SupportsAbsAndDunderGE
    ) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self,
        first: SupportsSub[_T, SupportsAbs[SupportsRound[object]]],
        second: _T,
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self,
        first: _T,
        second: SupportsRSub[_T, SupportsAbs[SupportsRound[object]]],
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    def assertRegex(self, text: AnyStr, expected_regex: AnyStr | Pattern[AnyStr], msg: Any = None) -> None: ...
    def assertNotRegex(self, text: AnyStr, unexpected_regex: AnyStr | Pattern[AnyStr], msg: Any = None) -> None: ...
    def assertCountEqual(self, first: Iterable[Any], second: Iterable[Any], msg: Any = None) -> None: ...
    def addTypeEqualityFunc(self, typeobj: type[Any], function: Callable[..., None]) -> None: ...
    def assertMultiLineEqual(self, first: str, second: str, msg: Any = None) -> None: ...
    def assertSequenceEqual(
        self, seq1: Sequence[Any], seq2: Sequence[Any], msg: Any = None, seq_type: type[Sequence[Any]] | None = None
    ) -> None: ...
    def assertListEqual(self, list1: list[Any], list2: list[Any], msg: Any = None) -> None: ...
    def assertTupleEqual(self, tuple1: tuple[Any, ...], tuple2: tuple[Any, ...], msg: Any = None) -> None: ...
    def assertSetEqual(self, set1: AbstractSet[object], set2: AbstractSet[object], msg: Any = None) -> None: ...
    def assertDictEqual(self, d1: Mapping[Any, object], d2: Mapping[Any, object], msg: Any = None) -> None: ...
    def fail(self, msg: Any = None) -> NoReturn: ...
    def countTestCases(self) -> int: ...
    def defaultTestResult(self) -> unittest.result.TestResult: ...
    def id(self) -> str: ...
    def shortDescription(self) -> str | None: ...
    if sys.version_info >= (3, 8):
        def addCleanup(self, __function: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    else:
        def addCleanup(self, function: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs) -> None: ...

    if sys.version_info >= (3, 11):
        def enterContext(self, cm: AbstractContextManager[_T]) -> _T: ...

    def doCleanups(self) -> None: ...
    if sys.version_info >= (3, 8):
        @classmethod
        def addClassCleanup(cls, __function: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs) -> None: ...
        @classmethod
        def doClassCleanups(cls) -> None: ...

    if sys.version_info >= (3, 11):
        @classmethod
        def enterClassContext(cls, cm: AbstractContextManager[_T]) -> _T: ...

    def _formatMessage(self, msg: str | None, standardMsg: str) -> str: ...  # undocumented
    def _getAssertEqualityFunc(self, first: Any, second: Any) -> Callable[..., None]: ...  # undocumented
    if sys.version_info < (3, 12):
        failUnlessEqual = assertEqual
        assertEquals = assertEqual
        failIfEqual = assertNotEqual
        assertNotEquals = assertNotEqual
        failUnless = assertTrue
        assert_ = assertTrue
        failIf = assertFalse
        failUnlessRaises = assertRaises
        failUnlessAlmostEqual = assertAlmostEqual
        assertAlmostEquals = assertAlmostEqual
        failIfAlmostEqual = assertNotAlmostEqual
        assertNotAlmostEquals = assertNotAlmostEqual
        assertRegexpMatches = assertRegex
        assertNotRegexpMatches = assertNotRegex
        assertRaisesRegexp = assertRaisesRegex
        def assertDictContainsSubset(
            self, subset: Mapping[Any, Any], dictionary: Mapping[Any, Any], msg: object = None
        ) -> None: ...

class FunctionTestCase(TestCase):
    def __init__(
        self,
        testFunc: Callable[[], object],
        setUp: Callable[[], object] | None = None,
        tearDown: Callable[[], object] | None = None,
        description: str | None = None,
    ) -> None: ...
    def runTest(self) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...

class _AssertRaisesContext(Generic[_E]):
    exception: _E
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, tb: TracebackType | None
    ) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class _AssertWarnsContext:
    warning: WarningMessage
    filename: str
    lineno: int
    warnings: list[WarningMessage]
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, tb: TracebackType | None
    ) -> None: ...



import sys
from _typeshed import structseq
from typing import Any, Final, final

if sys.platform != "win32":
    @final
    class struct_spwd(structseq[Any], tuple[str, str, int, int, int, int, int, int, int]):
        if sys.version_info >= (3, 10):
            __match_args__: Final = (
                "sp_namp",
                "sp_pwdp",
                "sp_lstchg",
                "sp_min",
                "sp_max",
                "sp_warn",
                "sp_inact",
                "sp_expire",
                "sp_flag",
            )

        @property
        def sp_namp(self) -> str: ...
        @property
        def sp_pwdp(self) -> str: ...
        @property
        def sp_lstchg(self) -> int: ...
        @property
        def sp_min(self) -> int: ...
        @property
        def sp_max(self) -> int: ...
        @property
        def sp_warn(self) -> int: ...
        @property
        def sp_inact(self) -> int: ...
        @property
        def sp_expire(self) -> int: ...
        @property
        def sp_flag(self) -> int: ...

    def getspall() -> list[struct_spwd]: ...
    def getspnam(arg: str, /) -> struct_spwd: ...



import sys
import types
from _typeshed import IdentityFunction, SupportsAllComparisons, SupportsItems
from collections.abc import Callable, Hashable, Iterable, Sequence, Sized
from typing import Any, Generic, NamedTuple, TypeVar, overload
from typing_extensions import Literal, Self, TypeAlias, TypedDict, final

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "update_wrapper",
    "wraps",
    "WRAPPER_ASSIGNMENTS",
    "WRAPPER_UPDATES",
    "total_ordering",
    "cmp_to_key",
    "lru_cache",
    "reduce",
    "partial",
    "partialmethod",
    "singledispatch",
]

if sys.version_info >= (3, 8):
    __all__ += ["cached_property", "singledispatchmethod"]

if sys.version_info >= (3, 9):
    __all__ += ["cache"]

_AnyCallable: TypeAlias = Callable[..., object]

_T = TypeVar("_T")
_S = TypeVar("_S")

@overload
def reduce(__function: Callable[[_T, _S], _T], __sequence: Iterable[_S], __initial: _T) -> _T: ...
@overload
def reduce(__function: Callable[[_T, _T], _T], __sequence: Iterable[_T]) -> _T: ...

class _CacheInfo(NamedTuple):
    hits: int
    misses: int
    maxsize: int | None
    currsize: int

if sys.version_info >= (3, 9):
    class _CacheParameters(TypedDict):
        maxsize: int
        typed: bool

@final
class _lru_cache_wrapper(Generic[_T]):
    __wrapped__: Callable[..., _T]
    def __call__(self, *args: Hashable, **kwargs: Hashable) -> _T: ...
    def cache_info(self) -> _CacheInfo: ...
    def cache_clear(self) -> None: ...
    if sys.version_info >= (3, 9):
        def cache_parameters(self) -> _CacheParameters: ...

    def __copy__(self) -> _lru_cache_wrapper[_T]: ...
    def __deepcopy__(self, __memo: Any) -> _lru_cache_wrapper[_T]: ...

if sys.version_info >= (3, 8):
    @overload
    def lru_cache(maxsize: int | None = 128, typed: bool = False) -> Callable[[Callable[..., _T]], _lru_cache_wrapper[_T]]: ...
    @overload
    def lru_cache(maxsize: Callable[..., _T], typed: bool = False) -> _lru_cache_wrapper[_T]: ...

else:
    def lru_cache(maxsize: int | None = 128, typed: bool = False) -> Callable[[Callable[..., _T]], _lru_cache_wrapper[_T]]: ...

if sys.version_info >= (3, 12):
    WRAPPER_ASSIGNMENTS: tuple[
        Literal["__module__"],
        Literal["__name__"],
        Literal["__qualname__"],
        Literal["__doc__"],
        Literal["__annotations__"],
        Literal["__type_params__"],
    ]
else:
    WRAPPER_ASSIGNMENTS: tuple[
        Literal["__module__"], Literal["__name__"], Literal["__qualname__"], Literal["__doc__"], Literal["__annotations__"]
    ]
WRAPPER_UPDATES: tuple[Literal["__dict__"]]

if sys.version_info >= (3, 12):
    def update_wrapper(
        wrapper: _T,
        wrapped: _AnyCallable,
        assigned: Sequence[str] = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__", "__type_params__"),
        updated: Sequence[str] = ("__dict__",),
    ) -> _T: ...
    def wraps(
        wrapped: _AnyCallable,
        assigned: Sequence[str] = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__", "__type_params__"),
        updated: Sequence[str] = ("__dict__",),
    ) -> IdentityFunction: ...

else:
    def update_wrapper(
        wrapper: _T,
        wrapped: _AnyCallable,
        assigned: Sequence[str] = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__"),
        updated: Sequence[str] = ("__dict__",),
    ) -> _T: ...
    def wraps(
        wrapped: _AnyCallable,
        assigned: Sequence[str] = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__"),
        updated: Sequence[str] = ("__dict__",),
    ) -> IdentityFunction: ...

def total_ordering(cls: type[_T]) -> type[_T]: ...
def cmp_to_key(mycmp: Callable[[_T, _T], int]) -> Callable[[_T], SupportsAllComparisons]: ...

class partial(Generic[_T]):
    @property
    def func(self) -> Callable[..., _T]: ...
    @property
    def args(self) -> tuple[Any, ...]: ...
    @property
    def keywords(self) -> dict[str, Any]: ...
    def __new__(cls, __func: Callable[..., _T], *args: Any, **kwargs: Any) -> Self: ...
    def __call__(__self, *args: Any, **kwargs: Any) -> _T: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

# With protocols, this could change into a generic protocol that defines __get__ and returns _T
_Descriptor: TypeAlias = Any

class partialmethod(Generic[_T]):
    func: Callable[..., _T] | _Descriptor
    args: tuple[Any, ...]
    keywords: dict[str, Any]
    @overload
    def __init__(self, __func: Callable[..., _T], *args: Any, **keywords: Any) -> None: ...
    @overload
    def __init__(self, __func: _Descriptor, *args: Any, **keywords: Any) -> None: ...
    if sys.version_info >= (3, 8):
        def __get__(self, obj: Any, cls: type[Any] | None = None) -> Callable[..., _T]: ...
    else:
        def __get__(self, obj: Any, cls: type[Any] | None) -> Callable[..., _T]: ...

    @property
    def __isabstractmethod__(self) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class _SingleDispatchCallable(Generic[_T]):
    registry: types.MappingProxyType[Any, Callable[..., _T]]
    def dispatch(self, cls: Any) -> Callable[..., _T]: ...
    # @fun.register(complex)
    # def _(arg, verbose=False): ...
    @overload
    def register(self, cls: type[Any], func: None = None) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...
    # @fun.register
    # def _(arg: int, verbose=False):
    @overload
    def register(self, cls: Callable[..., _T], func: None = None) -> Callable[..., _T]: ...
    # fun.register(int, lambda x: x)
    @overload
    def register(self, cls: type[Any], func: Callable[..., _T]) -> Callable[..., _T]: ...
    def _clear_cache(self) -> None: ...
    def __call__(__self, *args: Any, **kwargs: Any) -> _T: ...

def singledispatch(func: Callable[..., _T]) -> _SingleDispatchCallable[_T]: ...

if sys.version_info >= (3, 8):
    class singledispatchmethod(Generic[_T]):
        dispatcher: _SingleDispatchCallable[_T]
        func: Callable[..., _T]
        def __init__(self, func: Callable[..., _T]) -> None: ...
        @property
        def __isabstractmethod__(self) -> bool: ...
        @overload
        def register(self, cls: type[Any], method: None = None) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...
        @overload
        def register(self, cls: Callable[..., _T], method: None = None) -> Callable[..., _T]: ...
        @overload
        def register(self, cls: type[Any], method: Callable[..., _T]) -> Callable[..., _T]: ...
        def __get__(self, obj: _S, cls: type[_S] | None = None) -> Callable[..., _T]: ...

    class cached_property(Generic[_T]):
        func: Callable[[Any], _T]
        attrname: str | None
        def __init__(self, func: Callable[[Any], _T]) -> None: ...
        @overload
        def __get__(self, instance: None, owner: type[Any] | None = None) -> Self: ...
        @overload
        def __get__(self, instance: object, owner: type[Any] | None = None) -> _T: ...
        def __set_name__(self, owner: type[Any], name: str) -> None: ...
        # __set__ is not defined at runtime, but @cached_property is designed to be settable
        def __set__(self, instance: object, value: _T) -> None: ...
        if sys.version_info >= (3, 9):
            def __class_getitem__(cls, item: Any) -> GenericAlias: ...

if sys.version_info >= (3, 9):
    def cache(__user_function: Callable[..., _T]) -> _lru_cache_wrapper[_T]: ...

def _make_key(
    args: tuple[Hashable, ...],
    kwds: SupportsItems[Any, Any],
    typed: bool,
    kwd_mark: tuple[object, ...] = ...,
    fasttypes: set[type] = ...,
    tuple: type = ...,
    type: Any = ...,
    len: Callable[[Sized], int] = ...,
) -> Hashable: ...



from typing import Any

from django.contrib.gis.gdal.base import GDALBase

class GDALRasterBase(GDALBase):
    @property
    def metadata(self) -> Any: ...
    @metadata.setter
    def metadata(self, value: Any) -> None: ...



from collections.abc import Sequence
from email import charset as Charset
from email._policybase import Policy
from email.message import Message
from email.mime.base import MIMEBase
from email.mime.message import MIMEMessage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

# switch to tuple once https://github.com/python/mypy/issues/11098 is fixed
# remove Optional once python 3.7 is dropped (Tuple[str | None, ...] works with mypy on py3.10)
from typing import Any, Optional, Tuple, overload  # noqa: Y022, Y037

from typing_extensions import TypeAlias

utf8_charset: Any
utf8_charset_qp: Any
DEFAULT_ATTACHMENT_MIME_TYPE: str
RFC5322_EMAIL_LINE_LENGTH_LIMIT: int

class BadHeaderError(ValueError): ...

ADDRESS_HEADERS: set[str]

def forbid_multi_line_headers(name: str, val: str, encoding: str) -> tuple[str, str]: ...
def sanitize_address(addr: tuple[str, str] | str, encoding: str) -> str: ...

class MIMEMixin:
    def as_string(self, unixfrom: bool = ..., linesep: str = ...) -> str: ...
    def as_bytes(self, unixfrom: bool = ..., linesep: str = ...) -> bytes: ...

class SafeMIMEMessage(MIMEMixin, MIMEMessage):  # type: ignore
    defects: list[Any]
    epilogue: Any
    policy: Policy
    preamble: Any
    def __setitem__(self, name: str, val: str) -> None: ...

class SafeMIMEText(MIMEMixin, MIMEText):  # type: ignore
    defects: list[Any]
    epilogue: None
    policy: Policy
    preamble: None
    encoding: str
    def __init__(self, _text: str, _subtype: str = ..., _charset: str = ...) -> None: ...
    def __setitem__(self, name: str, val: str) -> None: ...
    def set_payload(
        self, payload: list[Message] | str | bytes, charset: str | Charset.Charset | None = ...
    ) -> None: ...

class SafeMIMEMultipart(MIMEMixin, MIMEMultipart):  # type: ignore
    defects: list[Any]
    epilogue: None
    policy: Policy
    preamble: None
    encoding: str
    def __init__(
        self,
        _subtype: str = ...,
        boundary: Any | None = ...,
        _subparts: Any | None = ...,
        encoding: str = ...,
        **_params: Any
    ) -> None: ...
    def __setitem__(self, name: str, val: str) -> None: ...

_AttachmentContent: TypeAlias = bytes | EmailMessage | Message | SafeMIMEText | str
# switch to tuple once https://github.com/python/mypy/issues/11098 is fixed
# remove Optional once python 3.7 is dropped (Tuple[str | None, ...] works with mypy on py3.10)
_AttachmentTuple: TypeAlias = (
    Tuple[str, _AttachmentContent]
    | Tuple[Optional[str], _AttachmentContent, str]
    | Tuple[str, _AttachmentContent, None]
)

class EmailMessage:
    content_subtype: str
    mixed_subtype: str
    encoding: Any
    to: list[str]
    cc: list[Any]
    bcc: list[Any]
    reply_to: list[Any]
    from_email: str
    subject: str
    body: str
    attachments: list[Any]
    extra_headers: dict[Any, Any]
    connection: Any
    def __init__(
        self,
        subject: str = ...,
        body: str | None = ...,
        from_email: str | None = ...,
        to: Sequence[str] | None = ...,
        bcc: Sequence[str] | None = ...,
        connection: Any | None = ...,
        attachments: Sequence[MIMEBase | _AttachmentTuple] | None = ...,
        headers: dict[str, str] | None = ...,
        cc: Sequence[str] | None = ...,
        reply_to: Sequence[str] | None = ...,
    ) -> None: ...
    def get_connection(self, fail_silently: bool = ...) -> Any: ...
    # TODO: when typeshed gets more types for email.Message, move it to MIMEMessage, now it has too many false-positives
    def message(self) -> Any: ...
    def recipients(self) -> list[str]: ...
    def send(self, fail_silently: bool = ...) -> int: ...
    @overload
    def attach(self, filename: MIMEBase = ..., content: None = ..., mimetype: None = ...) -> None: ...
    @overload
    def attach(self, filename: None = ..., content: _AttachmentContent = ..., mimetype: str = ...) -> None: ...
    @overload
    def attach(self, filename: str = ..., content: _AttachmentContent = ..., mimetype: str | None = ...) -> None: ...
    def attach_file(self, path: str, mimetype: str | None = ...) -> None: ...

class EmailMultiAlternatives(EmailMessage):
    alternative_subtype: str
    alternatives: list[tuple[_AttachmentContent, str]]
    def __init__(
        self,
        subject: str = ...,
        body: str | None = ...,
        from_email: str | None = ...,
        to: Sequence[str] | None = ...,
        bcc: Sequence[str] | None = ...,
        connection: Any | None = ...,
        attachments: Sequence[MIMEBase | _AttachmentTuple] | None = ...,
        headers: dict[str, str] | None = ...,
        alternatives: list[tuple[_AttachmentContent, str]] | None = ...,
        cc: Sequence[str] | None = ...,
        reply_to: Sequence[str] | None = ...,
    ) -> None: ...
    def attach_alternative(self, content: _AttachmentContent, mimetype: str) -> None: ...



from _typeshed import StrPath
from collections.abc import Callable
from typing import Any, BinaryIO, Protocol, overload

__all__ = ["what"]

class _ReadableBinary(Protocol):
    def tell(self) -> int: ...
    def read(self, __size: int) -> bytes: ...
    def seek(self, __offset: int) -> Any: ...

@overload
def what(file: StrPath | _ReadableBinary, h: None = None) -> str | None: ...
@overload
def what(file: Any, h: bytes) -> str | None: ...

tests: list[Callable[[bytes, BinaryIO | None], str | None]]



from collections.abc import Callable, Sequence
from typing import IO, Any
from typing_extensions import TypeAlias

__all__ = ["Timer", "timeit", "repeat", "default_timer"]

_Timer: TypeAlias = Callable[[], float]
_Stmt: TypeAlias = str | Callable[[], object]

default_timer: _Timer

class Timer:
    def __init__(
        self, stmt: _Stmt = "pass", setup: _Stmt = "pass", timer: _Timer = ..., globals: dict[str, Any] | None = None
    ) -> None: ...
    def print_exc(self, file: IO[str] | None = None) -> None: ...
    def timeit(self, number: int = 1000000) -> float: ...
    def repeat(self, repeat: int = 5, number: int = 1000000) -> list[float]: ...
    def autorange(self, callback: Callable[[int, float], object] | None = None) -> tuple[int, float]: ...

def timeit(
    stmt: _Stmt = "pass", setup: _Stmt = "pass", timer: _Timer = ..., number: int = 1000000, globals: dict[str, Any] | None = None
) -> float: ...
def repeat(
    stmt: _Stmt = "pass",
    setup: _Stmt = "pass",
    timer: _Timer = ...,
    repeat: int = 5,
    number: int = 1000000,
    globals: dict[str, Any] | None = None,
) -> list[float]: ...
def main(args: Sequence[str] | None = None, *, _wrap_timer: Callable[[_Timer], _Timer] | None = None) -> None: ...



import tempfile
from typing import Optional


def get_random_tempfile_path(suffix: Optional[str] = None, delete: bool = False) -> str:
    """
    Creates a temporary file and returns the path of it.

    The file is located in the temp folder of your operating system.
    """

    if suffix is not None:
        temp_file = tempfile.NamedTemporaryFile(delete=delete, suffix=f".{suffix}")
    else:
        temp_file = tempfile.NamedTemporaryFile(
            delete=delete,
        )

    return temp_file.name


__all__ = ["get_random_tempfile_path"]



from django.conf import settings
from pydantic import ValidationError

from nwon_django_toolbox.nwon_django_settings import NWONDjangoSettings


def set_settings() -> NWONDjangoSettings:
    """
    Parse Settings from Django settings
    """

    if not hasattr(settings, "NWON_DJANGO"):
        return NWONDjangoSettings()

    if isinstance(settings.NWON_DJANGO, NWONDjangoSettings):
        return settings.NWON_DJANGO

    if not isinstance(settings.NWON_DJANGO, dict):
        raise Exception(
            "The NWON_DJANGO settings need to be of type dict or NWONDjangoSettings"
        )

    try:
        return NWONDjangoSettings.model_validate(settings.NWON_DJANGO)
    except ValidationError as exception:
        raise Exception(
            f"Could not parse the NWON_DJANGO settings: {str(exception)}"
        ) from exception


NWON_DJANGO_SETTINGS = set_settings()
"""
Settings used withing the NWON-django-toolbox package
"""

__all__ = ["NWON_DJANGO_SETTINGS"]



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .. import fixer_base

iter_exempt: set[str]

class FixDict(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    def transform(self, node, results): ...
    P1: ClassVar[str]
    p1: ClassVar[Incomplete]
    P2: ClassVar[str]
    p2: ClassVar[Incomplete]
    def in_special_context(self, node, isiter): ...



from collections.abc import Callable, Iterable
from typing import TypeVar

from django.utils.deprecation import MiddlewareMixin
from django.utils.functional import classproperty as classproperty
from django.views.generic.base import View

_T = TypeVar("_T", bound=View | Callable)  # Any callable
_CallableType = TypeVar("_CallableType", bound=Callable)

class classonlymethod(classmethod): ...

def method_decorator(decorator: Callable | Iterable[Callable], name: str = ...) -> Callable[[_T], _T]: ...
def decorator_from_middleware_with_args(middleware_class: type) -> Callable: ...
def decorator_from_middleware(middleware_class: type) -> Callable: ...
def make_middleware_decorator(middleware_class: type[MiddlewareMixin]) -> Callable: ...
def sync_and_async_middleware(func: _CallableType) -> _CallableType: ...
def sync_only_middleware(func: _CallableType) -> _CallableType: ...
def async_only_middleware(func: _CallableType) -> _CallableType: ...



from collections.abc import Generator, Iterable
from typing import ClassVar, TypeVar
from typing_extensions import Literal

from .. import fixer_base
from ..pytree import Base

_N = TypeVar("_N", bound=Base)

def find_excepts(nodes: Iterable[_N]) -> Generator[tuple[_N, _N], None, None]: ...

class FixExcept(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    def transform(self, node, results): ...



from collections.abc import Callable
from typing import Any, TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

def cache_page(timeout: float | None, *, cache: Any | None = ..., key_prefix: Any | None = ...) -> Callable: ...
def cache_control(**kwargs: Any) -> Callable: ...
def never_cache(view_func: _F) -> _F: ...



from typing import Any

from django.contrib.gis.db.backends.base.models import SpatialRefSysMixin
from django.db import models

class SpatialiteGeometryColumns(models.Model):
    f_table_name: Any
    f_geometry_column: Any
    coord_dimension: Any
    srid: Any
    spatial_index_enabled: Any
    type: Any

    class Meta:
        app_label: str
        db_table: str
        managed: bool
    @classmethod
    def table_name_col(cls) -> Any: ...
    @classmethod
    def geom_col_name(cls) -> Any: ...

class SpatialiteSpatialRefSys(models.Model, SpatialRefSysMixin):
    srid: Any
    auth_name: Any
    auth_srid: Any
    ref_sys_name: Any
    proj4text: Any
    srtext: Any

    class Meta:
        app_label: str
        db_table: str
        managed: bool
    @property
    def wkt(self) -> Any: ...



from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Aggregate
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType

class GeoAggregate(Aggregate):
    is_extent: bool
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Collect(GeoAggregate):
    name: str
    output_field_class: Any

class Extent(GeoAggregate):
    name: str
    def __init__(self, expression: Any, **extra: Any) -> None: ...

class Extent3D(GeoAggregate):
    name: str
    def __init__(self, expression: Any, **extra: Any) -> None: ...

class MakeLine(GeoAggregate):
    name: str
    output_field_class: Any

class Union(GeoAggregate):
    name: str
    output_field_class: Any



from collections.abc import Callable
from datetime import date, datetime
from typing import Any, SupportsInt

from django import template

register: template.Library

def ordinal(value: str | SupportsInt | None) -> str | None: ...
def intcomma(value: str | SupportsInt | None, use_l10n: bool = ...) -> str: ...

intword_converters: tuple[tuple[int, Callable]]

def intword(value: str | SupportsInt | None) -> int | str | None: ...
def apnumber(value: str | SupportsInt | None) -> int | str | None: ...
def naturalday(value: date | str | None, arg: str | None = ...) -> str | None: ...
def naturaltime(value: datetime) -> str: ...

class NaturalTimeFormatter:
    time_strings: dict[str, str]
    past_substrings: dict[str, str]
    future_substrings: dict[str, str]
    @classmethod
    def string_for(cls: type[NaturalTimeFormatter], value: Any) -> Any: ...



from nwon_django_toolbox.tests.api_client import ApiClient, ApiTest
from nwon_django_toolbox.tests.api_method_tests import (
    check_delete_basics,
    check_delete_not_allowed,
    check_delete_permissions,
    check_get_basics,
    check_get_detail_permissions,
    check_get_list_permissions,
    check_patch_basics,
    check_patch_method_not_allowed,
    check_patch_parameters_failing,
    check_patch_parameters_successful,
    check_patch_permissions,
    check_patch_read_only_field,
    check_permission,
    check_permissions,
    check_post_basics,
    check_post_not_allowed,
    check_post_parameters_failing,
    check_post_parameters_not_required,
    check_post_parameters_required,
    check_post_parameters_successful,
    check_post_permissions,
    check_post_read_only_field,
    check_put_basics,
    check_put_not_allowed,
    check_put_parameters_failing,
    check_put_parameters_successful,
    check_put_permissions,
    check_put_read_only_field,
    ensure_key_with_object_list,
    ensure_paged_results,
)
from nwon_django_toolbox.tests.base_meta_factory import BaseMetaFactory
from nwon_django_toolbox.tests.celery import (
    check_latest_task,
    check_number_of_created_tasks,
    check_task_has_been_enqueued,
    clean_all_celery_messages,
    create_celery_folder,
    get_path_to_celery_folder,
)
from nwon_django_toolbox.tests.file_exists_in_storage import file_exists_in_storage
from nwon_django_toolbox.tests.helper import (
    check_object_against_parameter,
    dictionary_is_serialized_instance,
    get_path_to_random_image,
    get_random_image,
    get_random_tempfile_path,
)

__all__ = [
    "file_exists_in_storage",
    "check_latest_task",
    "check_number_of_created_tasks",
    "check_task_has_been_enqueued",
    "clean_all_celery_messages",
    "create_celery_folder",
    "get_path_to_celery_folder",
    "get_path_to_random_image",
    "get_random_image",
    "get_random_tempfile_path",
    "check_latest_task",
    "check_number_of_created_tasks",
    "check_task_has_been_enqueued",
    "clean_all_celery_messages",
    "create_celery_folder",
    "get_path_to_celery_folder",
    "check_delete_basics",
    "check_delete_not_allowed",
    "check_get_basics",
    "check_patch_basics",
    "check_patch_method_not_allowed",
    "check_patch_parameters_failing",
    "check_patch_parameters_successful",
    "check_patch_read_only_field",
    "check_post_basics",
    "check_post_not_allowed",
    "check_post_parameters_failing",
    "check_post_parameters_not_required",
    "check_post_parameters_successful",
    "check_post_read_only_field",
    "check_put_basics",
    "check_put_not_allowed",
    "check_put_parameters_failing",
    "check_put_parameters_successful",
    "check_put_read_only_field",
    "check_object_against_parameter",
    "ApiClient",
    "ApiTest",
    "ensure_paged_results",
    "ensure_key_with_object_list",
    "check_delete_permissions",
    "check_get_detail_permissions",
    "check_get_list_permissions",
    "check_patch_permissions",
    "check_post_parameters_required",
    "check_permission",
    "check_permissions",
    "check_post_permissions",
    "check_put_permissions",
    "dictionary_is_serialized_instance",
]



from typing import ClassVar
from typing_extensions import Literal

from .. import fixer_base

class FixIsinstance(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    def transform(self, node, results) -> None: ...



from typing import ClassVar
from typing_extensions import Literal

from .. import fixer_base

class FixNumliterals(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[False]]
    def match(self, node): ...
    def transform(self, node, results): ...



from collections.abc import Iterator
from typing import IO, Any

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer
from django.db.models.fields import Field
from yaml import CSafeDumper as SafeDumper
from yaml import MappingNode, ScalarNode

class DjangoSafeDumper(SafeDumper):
    def represent_decimal(self, data: Any) -> ScalarNode: ...
    def represent_ordered_dict(self, data: Any) -> MappingNode: ...

class Serializer(PythonSerializer):
    internal_use_only: bool
    def handle_field(self, obj: Any, field: Field) -> None: ...
    def end_serialization(self) -> None: ...
    def getvalue(self) -> Any: ...

def Deserializer(
    stream_or_string: bytes | str | IO[bytes] | IO[str], **options: Any
) -> Iterator[DeserializedObject]: ...



from typing import Any

from django.contrib.gis.gdal import DataSource

def mapping(
    data_source: str | DataSource, geom_name: str = ..., layer_key: int | str = ..., multi_geom: bool = ...
) -> dict[str, str]: ...
def ogrinspect(data_source: str | DataSource, model_name: str, *args: Any, **kwargs: Any) -> str: ...



from typing import Any

from django.core.management.base import BaseCommand

OPENAPI_MODE: str
COREAPI_MODE: str

class Command(BaseCommand):
    help: str
    def get_mode(self): ...
    def add_arguments(self, parser: Any) -> None: ...
    def handle(self, *args: Any, **options: Any) -> None: ...
    def get_renderer(self, format: str): ...
    def get_generator_class(self): ...



from collections.abc import Callable
from typing import Any

from django.core.cache.backends.base import BaseCache
from rest_framework.request import Request
from rest_framework.views import APIView

class BaseThrottle:
    def allow_request(self, request: Request, view: APIView) -> bool: ...
    def get_ident(self, request: Request) -> str: ...
    def wait(self) -> float | None: ...

class SimpleRateThrottle(BaseThrottle):
    cache: BaseCache
    cache_format: str
    history: list[Any]
    key: str | None
    now: float
    rate: str | None
    scope: str | None
    THROTTLE_RATES: dict[str, str | None]
    timer: Callable[..., float]
    def __init__(self) -> None: ...
    def get_cache_key(self, request: Request, view: APIView) -> str | None: ...
    def get_rate(self) -> str | None: ...
    def parse_rate(self, rate) -> tuple[int | None, int | None]: ...
    def throttle_failure(self) -> bool: ...
    def throttle_success(self) -> bool: ...

class AnonRateThrottle(SimpleRateThrottle): ...
class UserRateThrottle(SimpleRateThrottle): ...

class ScopedRateThrottle(SimpleRateThrottle):
    scope_attr: str



import copy
from typing import Any, List, Optional, Type

from jsonschema_to_openapi.convert import convert as convert_to_openapi
from nwon_baseline.pydantic import pydantic_model_to_dict, schema_from_pydantic_model
from nwon_baseline.typings import AnyDict
from pydantic import ValidationError as PydanticValidationError
from pydantic.main import BaseModel
from rest_framework.exceptions import ValidationError
from rest_framework.serializers import JSONField

__all__ = ["PydanticJsonFieldSerializer"]


class PydanticJsonFieldSerializer(JSONField):
    """
    Serializer for serializing our custom PydanticJsonField.

    Provides annotations for both drf-spectacular and drf-yasg
    """

    class Meta:
        swagger_schema_fields: AnyDict

    def __init__(
        self, *args, pydantic_models: Optional[List[Type[BaseModel]]] = None, **kwargs
    ):
        super().__init__(*args, **kwargs)

        # Convert list to tuple for caching
        models_tuple = tuple(pydantic_models) if pydantic_models else None
        self.schema = self.__schema_information(models_tuple)

        # Set schema for drf-spectacular
        self.coreapi_schema = convert_to_openapi(copy.deepcopy(self.schema))

        # Set schema for drf-yasg
        PydanticJsonFieldSerializer.Meta.swagger_schema_fields = self.schema

        self.pydantic_models = pydantic_models if pydantic_models else []

    def to_representation(self, value: Any):
        value = super().to_representation(value)

        if value is None:
            return None

        for model in self.pydantic_models:
            try:
                if isinstance(value, dict):
                    return pydantic_model_to_dict(model.model_validate(value))
                else:
                    return pydantic_model_to_dict(model.model_validate_strings(value))
            except PydanticValidationError as exc:
                print(f"Pydantic model validation failed: {exc}")

        # Return raw value or raise error if no model validated
        raise ValidationError("No matching Pydantic model for representation.")

    def to_internal_value(self, data: Any):
        data = super().to_internal_value(data)

        for model in self.pydantic_models:
            try:
                parsed_json = model.model_validate(data)
                return parsed_json.model_dump()
            except PydanticValidationError as error:
                error_details = [
                    {
                        "location": e["loc"],
                        "message": e["msg"],
                        "type": e.get("type", "unknown"),
                    }
                    for e in error.errors()
                ]

                raise ValidationError({"errors": error_details}) from error

        return None

    def __schema_information(
        self, pydantic_models: Optional[tuple[Type[BaseModel], ...]]
    ) -> AnyDict:
        """
        Returns a JSON schema that is used for representing the potential values of this field
        """

        if pydantic_models is None or len(pydantic_models) == 0:
            return {"type": "object"}

        elif len(pydantic_models) > 1:
            return {
                "anyOf": [
                    schema_from_pydantic_model(model) for model in pydantic_models
                ]
            }

        return (
            schema_from_pydantic_model(pydantic_models[0])
            if len(pydantic_models) > 0 and pydantic_models[0]
            else {"type": "object"}
        )


try:
    from drf_spectacular.extensions import OpenApiSerializerFieldExtension

    class PydanticJsonFieldSerializerExtension(OpenApiSerializerFieldExtension):  # type: ignore
        target_class = PydanticJsonFieldSerializer

        def map_serializer_field(self, auto_schema, direction):
            schema = self.target.schema or {"type": "object"}
            return schema

except Exception:
    pass



from humps.main import (
    camelize,
    decamelize,
    kebabize,
    dekebabize,
    depascalize,
    is_camelcase,
    is_pascalcase,
    is_kebabcase,
    is_snakecase,
    pascalize,
)

__all__ = [
    "camelize",
    "decamelize",
    "kebabize",
    "dekebabize",
    "depascalize",
    "is_camelcase",
    "is_pascalcase",
    "is_kebabcase",
    "is_snakecase",
    "pascalize",
]



from typing import Any, MutableMapping

class AttributeDictMixin:
    def __getattr__(self, k: str) -> Any: ...
    def __setattr__(self, key: str, value: Any) -> None: ...

class ChainMap(MutableMapping[str, Any]): ...  # type: ignore [misc]
class ConfigurationView(ChainMap, AttributeDictMixin): ...  # type: ignore [misc]



from collections.abc import Callable, Iterator
from email.message import Message
from typing import overload

__all__ = ["Charset", "add_alias", "add_charset", "add_codec"]

QP: int  # undocumented
BASE64: int  # undocumented
SHORTEST: int  # undocumented

class Charset:
    input_charset: str
    header_encoding: int
    body_encoding: int
    output_charset: str | None
    input_codec: str | None
    output_codec: str | None
    def __init__(self, input_charset: str = "us-ascii") -> None: ...
    def get_body_encoding(self) -> str | Callable[[Message], None]: ...
    def get_output_charset(self) -> str | None: ...
    def header_encode(self, string: str) -> str: ...
    def header_encode_lines(self, string: str, maxlengths: Iterator[int]) -> list[str | None]: ...
    @overload
    def body_encode(self, string: None) -> None: ...
    @overload
    def body_encode(self, string: str | bytes) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, __value: object) -> bool: ...

def add_charset(
    charset: str, header_enc: int | None = None, body_enc: int | None = None, output_charset: str | None = None
) -> None: ...
def add_alias(alias: str, canonical: str) -> None: ...
def add_codec(charset: str, codecname: str) -> None: ...



import codecs
import sys
from _typeshed import ReadableBuffer
from collections.abc import Callable
from typing import Literal, overload
from typing_extensions import TypeAlias

# This type is not exposed; it is defined in unicodeobject.c
class _EncodingMap:
    def size(self) -> int: ...

_CharMap: TypeAlias = dict[int, int] | _EncodingMap
_Handler: TypeAlias = Callable[[UnicodeError], tuple[str | bytes, int]]
_SearchFunction: TypeAlias = Callable[[str], codecs.CodecInfo | None]

def register(search_function: _SearchFunction, /) -> None: ...

if sys.version_info >= (3, 10):
    def unregister(search_function: _SearchFunction, /) -> None: ...

def register_error(errors: str, handler: _Handler, /) -> None: ...
def lookup_error(name: str, /) -> _Handler: ...

# The type ignore on `encode` and `decode` is to avoid issues with overlapping overloads, for more details, see #300
# https://docs.python.org/3/library/codecs.html#binary-transforms
_BytesToBytesEncoding: TypeAlias = Literal[
    "base64",
    "base_64",
    "base64_codec",
    "bz2",
    "bz2_codec",
    "hex",
    "hex_codec",
    "quopri",
    "quotedprintable",
    "quoted_printable",
    "quopri_codec",
    "uu",
    "uu_codec",
    "zip",
    "zlib",
    "zlib_codec",
]
# https://docs.python.org/3/library/codecs.html#text-transforms
_StrToStrEncoding: TypeAlias = Literal["rot13", "rot_13"]

@overload
def encode(obj: ReadableBuffer, encoding: _BytesToBytesEncoding, errors: str = "strict") -> bytes: ...
@overload
def encode(obj: str, encoding: _StrToStrEncoding, errors: str = "strict") -> str: ...  # type: ignore[overload-overlap]
@overload
def encode(obj: str, encoding: str = "utf-8", errors: str = "strict") -> bytes: ...
@overload
def decode(obj: ReadableBuffer, encoding: _BytesToBytesEncoding, errors: str = "strict") -> bytes: ...  # type: ignore[overload-overlap]
@overload
def decode(obj: str, encoding: _StrToStrEncoding, errors: str = "strict") -> str: ...

# these are documented as text encodings but in practice they also accept str as input
@overload
def decode(
    obj: str,
    encoding: Literal["unicode_escape", "unicode-escape", "raw_unicode_escape", "raw-unicode-escape"],
    errors: str = "strict",
) -> str: ...

# hex is officially documented as a bytes to bytes encoding, but it appears to also work with str
@overload
def decode(obj: str, encoding: Literal["hex", "hex_codec"], errors: str = "strict") -> bytes: ...
@overload
def decode(obj: ReadableBuffer, encoding: str = "utf-8", errors: str = "strict") -> str: ...
def lookup(encoding: str, /) -> codecs.CodecInfo: ...
def charmap_build(map: str, /) -> _CharMap: ...
def ascii_decode(data: ReadableBuffer, errors: str | None = None, /) -> tuple[str, int]: ...
def ascii_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def charmap_decode(data: ReadableBuffer, errors: str | None = None, mapping: _CharMap | None = None, /) -> tuple[str, int]: ...
def charmap_encode(str: str, errors: str | None = None, mapping: _CharMap | None = None, /) -> tuple[bytes, int]: ...
def escape_decode(data: str | ReadableBuffer, errors: str | None = None, /) -> tuple[str, int]: ...
def escape_encode(data: bytes, errors: str | None = None, /) -> tuple[bytes, int]: ...
def latin_1_decode(data: ReadableBuffer, errors: str | None = None, /) -> tuple[str, int]: ...
def latin_1_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...

if sys.version_info >= (3, 9):
    def raw_unicode_escape_decode(
        data: str | ReadableBuffer, errors: str | None = None, final: bool = True, /
    ) -> tuple[str, int]: ...

else:
    def raw_unicode_escape_decode(data: str | ReadableBuffer, errors: str | None = None, /) -> tuple[str, int]: ...

def raw_unicode_escape_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def readbuffer_encode(data: str | ReadableBuffer, errors: str | None = None, /) -> tuple[bytes, int]: ...

if sys.version_info >= (3, 9):
    def unicode_escape_decode(
        data: str | ReadableBuffer, errors: str | None = None, final: bool = True, /
    ) -> tuple[str, int]: ...

else:
    def unicode_escape_decode(data: str | ReadableBuffer, errors: str | None = None, /) -> tuple[str, int]: ...

def unicode_escape_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_16_be_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_16_be_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_16_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_16_encode(str: str, errors: str | None = None, byteorder: int = 0, /) -> tuple[bytes, int]: ...
def utf_16_ex_decode(
    data: ReadableBuffer, errors: str | None = None, byteorder: int = 0, final: bool = False, /
) -> tuple[str, int, int]: ...
def utf_16_le_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_16_le_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_32_be_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_32_be_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_32_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_32_encode(str: str, errors: str | None = None, byteorder: int = 0, /) -> tuple[bytes, int]: ...
def utf_32_ex_decode(
    data: ReadableBuffer, errors: str | None = None, byteorder: int = 0, final: bool = False, /
) -> tuple[str, int, int]: ...
def utf_32_le_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_32_le_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_7_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_7_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
def utf_8_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
def utf_8_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...

if sys.platform == "win32":
    def mbcs_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
    def mbcs_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
    def code_page_decode(
        codepage: int, data: ReadableBuffer, errors: str | None = None, final: bool = False, /
    ) -> tuple[str, int]: ...
    def code_page_encode(code_page: int, str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...
    def oem_decode(data: ReadableBuffer, errors: str | None = None, final: bool = False, /) -> tuple[str, int]: ...
    def oem_encode(str: str, errors: str | None = None, /) -> tuple[bytes, int]: ...



from django.db.migrations.operations.base import Operation

class MigrationOptimizer:
    def optimize(self, operations: list[Operation], app_label: str | None) -> list[Operation]: ...
    def optimize_inner(self, operations: list[Operation], app_label: str | None) -> list[Operation]: ...



from django.core.signing import JSONSerializer as BaseJSONSerializer
from django.db.models.base import Model
from typing_extensions import TypeAlias

class PickleSerializer:
    def dumps(self, obj: dict[str, Model]) -> bytes: ...
    def loads(self, data: bytes) -> dict[str, Model]: ...

JSONSerializer: TypeAlias = BaseJSONSerializer



from typing import Any, Callable, Iterable, Mapping, Sequence, Tuple, Union

from celery.app.base import Celery
from kombu import Connection
from kombu.pidbox import Mailbox as KombuMailbox

_Reply = Any

_Destination = Union[Sequence[str], Tuple[str, ...]]

def flatten_reply(reply: Iterable[Mapping[str, _Reply]]) -> dict[str, _Reply]: ...

class Inspect:
    app: Celery | None
    def __init__(
        self,
        destination: _Destination | None = ...,
        timeout: float = ...,
        callback: Callable[..., Any] | None = ...,
        connection: Connection | None = ...,
        app: Celery | None = ...,
        limit: int | None = ...,
        pattern: str | None = ...,
        matcher: Callable[..., Any] | None = ...,
    ) -> None: ...
    def _prepare(self, reply: bool) -> None | dict[str, _Reply]: ...
    def _request(self, command: str, **kwargs: Any) -> list[dict[str, _Reply]]: ...
    def report(self) -> list[dict[str, Any]]: ...
    def clock(self) -> list[dict[str, Any]]: ...
    def active(self, safe: Any | None = ...) -> list[dict[str, _Reply]]: ...
    def scheduled(self, safe: Any | None = ...) -> list[dict[str, Any]]: ...
    def reserved(self, safe: Any | None = ...) -> list[dict[str, Any]]: ...
    def stats(self) -> dict[str, _Reply]: ...
    def revoked(self) -> list[dict[str, _Reply]]: ...
    def registered(self, *taskinfoitems: Any) -> list[dict[str, _Reply]]: ...
    registered_tasks = registered
    def ping(
        self, destination: _Destination | None = ...
    ) -> list[dict[str, _Reply]]: ...
    def active_queues(self) -> list[dict[str, _Reply]]: ...
    def query_task(self, *ids: str) -> list[dict[str, _Reply]]: ...
    def conf(self, with_defaults: bool = ...) -> list[dict[str, _Reply]]: ...
    def hello(
        self, from_node: Any, revoked: Any | None = ...
    ) -> list[dict[str, _Reply]]: ...
    def memsample(self) -> list[dict[str, _Reply]]: ...
    def memdump(self, samples: int = ...) -> list[dict[str, _Reply]]: ...
    def objgraph(
        self, type: str = ..., n: int = ..., max_depth: int = ...
    ) -> list[dict[str, _Reply]]: ...

class Control:
    Mailbox: KombuMailbox
    app: Celery | None
    def __init__(self, app: Celery | None = ...) -> None: ...
    def inspect(self) -> Any: ...
    def purge(self, connection: Connection | None = ...) -> int: ...
    discard_all = purge
    def election(
        self,
        id: str,
        topic: str,
        action: str | None = ...,
        connection: Connection | None = ...,
    ) -> None: ...
    def revoke(
        self,
        task_id: str | Sequence[str],
        destination: _Destination | None = ...,
        terminate: bool = ...,
        signal: str = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def terminate(
        self,
        task_id: str,
        destination: _Destination | None = ...,
        signal: str = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def ping(
        self,
        destination: _Destination | None = ...,
        timeout: float = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def rate_limit(
        self,
        task_name: str,
        rate_limit: int | str,
        destination: _Destination | None = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def add_consumer(
        self,
        queue: str,
        exchange: str | None = ...,
        exchange_type: str = ...,
        routing_key: str | None = ...,
        options: Mapping[str, Any] | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def cancel_consumer(
        self, queue: str, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def time_limit(
        self,
        task_name: str,
        soft: float | None = ...,
        hard: float | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def enable_events(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def disable_events(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_grow(
        self, n: int = ..., destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_shrink(
        self, n: int = ..., destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def autoscale(
        self, max: int, min: int, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def shutdown(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_restart(
        self,
        modules: Sequence[str] | None = ...,
        reload: bool = ...,
        reloader: Callable[..., Any] | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def heartbeat(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def broadcast(
        self,
        command: str,
        arguments: Mapping[str, Any] | None = ...,
        destination: _Destination | None = ...,
        connection: Connection | None = ...,
        reply: bool = ...,
        timeout: float = ...,
        limit: int | None = ...,
        callback: Callable[..., Any] | None = ...,
        channel: Any | None = ...,
        pattern: str | None = ...,
        matcher: Callable[..., Any] = ...,
        **extra_kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...



from collections.abc import Generator, Iterator, Sequence
from email import _ParamsType, _ParamType
from email.charset import Charset
from email.contentmanager import ContentManager
from email.errors import MessageDefect
from email.header import Header
from email.policy import Policy
from typing import Any, Literal, Protocol, TypeVar, overload
from typing_extensions import Self, TypeAlias

__all__ = ["Message", "EmailMessage"]

_T = TypeVar("_T")
_PayloadType: TypeAlias = Message | str
_EncodedPayloadType: TypeAlias = Message | bytes
_MultipartPayloadType: TypeAlias = list[_PayloadType]
_CharsetType: TypeAlias = Charset | str | None
# Type returned by Policy.header_fetch_parse, often str or Header.
_HeaderType: TypeAlias = Any
# Type accepted by Policy.header_store_parse.
_HeaderTypeParam: TypeAlias = str | Header | Any

class _SupportsEncodeToPayload(Protocol):
    def encode(self, encoding: str, /) -> _PayloadType | _MultipartPayloadType | _SupportsDecodeToPayload: ...

class _SupportsDecodeToPayload(Protocol):
    def decode(self, encoding: str, errors: str, /) -> _PayloadType | _MultipartPayloadType: ...

# TODO: This class should be generic over the header policy and/or the header
# value types allowed by the policy. This depends on PEP 696 support
# (https://github.com/python/typeshed/issues/11422).
class Message:
    policy: Policy  # undocumented
    preamble: str | None
    epilogue: str | None
    defects: list[MessageDefect]
    def is_multipart(self) -> bool: ...
    def set_unixfrom(self, unixfrom: str) -> None: ...
    def get_unixfrom(self) -> str | None: ...
    def attach(self, payload: _PayloadType) -> None: ...
    # `i: int` without a multipart payload results in an error
    # `| Any`: can be None for cleared or unset payload, but annoying to check
    @overload  # multipart
    def get_payload(self, i: int, decode: Literal[True]) -> None: ...
    @overload  # multipart
    def get_payload(self, i: int, decode: Literal[False] = False) -> _PayloadType | Any: ...
    @overload  # either
    def get_payload(self, i: None = None, decode: Literal[False] = False) -> _PayloadType | _MultipartPayloadType | Any: ...
    @overload  # not multipart
    def get_payload(self, i: None = None, *, decode: Literal[True]) -> _EncodedPayloadType | Any: ...
    @overload  # not multipart, IDEM but w/o kwarg
    def get_payload(self, i: None, decode: Literal[True]) -> _EncodedPayloadType | Any: ...
    # If `charset=None` and payload supports both `encode` AND `decode`, then an invalid payload could be passed, but this is unlikely
    # Not[_SupportsEncodeToPayload]
    @overload
    def set_payload(
        self, payload: _SupportsDecodeToPayload | _PayloadType | _MultipartPayloadType, charset: None = None
    ) -> None: ...
    @overload
    def set_payload(
        self,
        payload: _SupportsEncodeToPayload | _SupportsDecodeToPayload | _PayloadType | _MultipartPayloadType,
        charset: Charset | str,
    ) -> None: ...
    def set_charset(self, charset: _CharsetType) -> None: ...
    def get_charset(self) -> _CharsetType: ...
    def __len__(self) -> int: ...
    def __contains__(self, name: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    # Same as `get` with `failobj=None`, but with the expectation that it won't return None in most scenarios
    # This is important for protocols using __getitem__, like SupportsKeysAndGetItem
    # Morally, the return type should be `AnyOf[_HeaderType, None]`,
    # which we could spell as `_HeaderType | Any`,
    # *but* `_HeaderType` itself is currently an alias to `Any`...
    def __getitem__(self, name: str) -> _HeaderType: ...
    def __setitem__(self, name: str, val: _HeaderTypeParam) -> None: ...
    def __delitem__(self, name: str) -> None: ...
    def keys(self) -> list[str]: ...
    def values(self) -> list[_HeaderType]: ...
    def items(self) -> list[tuple[str, _HeaderType]]: ...
    @overload
    def get(self, name: str, failobj: None = None) -> _HeaderType | None: ...
    @overload
    def get(self, name: str, failobj: _T) -> _HeaderType | _T: ...
    @overload
    def get_all(self, name: str, failobj: None = None) -> list[_HeaderType] | None: ...
    @overload
    def get_all(self, name: str, failobj: _T) -> list[_HeaderType] | _T: ...
    def add_header(self, _name: str, _value: str, **_params: _ParamsType) -> None: ...
    def replace_header(self, _name: str, _value: _HeaderTypeParam) -> None: ...
    def get_content_type(self) -> str: ...
    def get_content_maintype(self) -> str: ...
    def get_content_subtype(self) -> str: ...
    def get_default_type(self) -> str: ...
    def set_default_type(self, ctype: str) -> None: ...
    @overload
    def get_params(
        self, failobj: None = None, header: str = "content-type", unquote: bool = True
    ) -> list[tuple[str, str]] | None: ...
    @overload
    def get_params(self, failobj: _T, header: str = "content-type", unquote: bool = True) -> list[tuple[str, str]] | _T: ...
    @overload
    def get_param(
        self, param: str, failobj: None = None, header: str = "content-type", unquote: bool = True
    ) -> _ParamType | None: ...
    @overload
    def get_param(self, param: str, failobj: _T, header: str = "content-type", unquote: bool = True) -> _ParamType | _T: ...
    def del_param(self, param: str, header: str = "content-type", requote: bool = True) -> None: ...
    def set_type(self, type: str, header: str = "Content-Type", requote: bool = True) -> None: ...
    @overload
    def get_filename(self, failobj: None = None) -> str | None: ...
    @overload
    def get_filename(self, failobj: _T) -> str | _T: ...
    @overload
    def get_boundary(self, failobj: None = None) -> str | None: ...
    @overload
    def get_boundary(self, failobj: _T) -> str | _T: ...
    def set_boundary(self, boundary: str) -> None: ...
    @overload
    def get_content_charset(self) -> str | None: ...
    @overload
    def get_content_charset(self, failobj: _T) -> str | _T: ...
    @overload
    def get_charsets(self, failobj: None = None) -> list[str | None]: ...
    @overload
    def get_charsets(self, failobj: _T) -> list[str | _T]: ...
    def walk(self) -> Generator[Self, None, None]: ...
    def get_content_disposition(self) -> str | None: ...
    def as_string(self, unixfrom: bool = False, maxheaderlen: int = 0, policy: Policy | None = None) -> str: ...
    def as_bytes(self, unixfrom: bool = False, policy: Policy | None = None) -> bytes: ...
    def __bytes__(self) -> bytes: ...
    def set_param(
        self,
        param: str,
        value: str,
        header: str = "Content-Type",
        requote: bool = True,
        charset: str | None = None,
        language: str = "",
        replace: bool = False,
    ) -> None: ...
    def __init__(self, policy: Policy = ...) -> None: ...
    # The following two methods are undocumented, but a source code comment states that they are public API
    def set_raw(self, name: str, value: _HeaderTypeParam) -> None: ...
    def raw_items(self) -> Iterator[tuple[str, _HeaderType]]: ...

class MIMEPart(Message):
    def __init__(self, policy: Policy | None = None) -> None: ...
    def get_body(self, preferencelist: Sequence[str] = ("related", "html", "plain")) -> Message | None: ...
    def iter_attachments(self) -> Iterator[Message]: ...
    def iter_parts(self) -> Iterator[Message]: ...
    def get_content(self, *args: Any, content_manager: ContentManager | None = None, **kw: Any) -> Any: ...
    def set_content(self, *args: Any, content_manager: ContentManager | None = None, **kw: Any) -> None: ...
    def make_related(self, boundary: str | None = None) -> None: ...
    def make_alternative(self, boundary: str | None = None) -> None: ...
    def make_mixed(self, boundary: str | None = None) -> None: ...
    def add_related(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def add_alternative(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def add_attachment(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def clear(self) -> None: ...
    def clear_content(self) -> None: ...
    def as_string(self, unixfrom: bool = False, maxheaderlen: int | None = None, policy: Policy | None = None) -> str: ...
    def is_attachment(self) -> bool: ...

class EmailMessage(MIMEPart): ...



from typing import Any

from ..cmd import Command

HAS_USER_SITE: bool
SCHEME_KEYS: tuple[str, ...]
INSTALL_SCHEMES: dict[str, dict[Any, Any]]

class install(Command):
    description: str
    user_options: Any
    boolean_options: Any
    negative_opt: Any
    prefix: str | None
    exec_prefix: Any
    home: str | None
    user: bool
    install_base: Any
    install_platbase: Any
    root: str | None
    install_purelib: Any
    install_platlib: Any
    install_headers: Any
    install_lib: str | None
    install_scripts: Any
    install_data: Any
    install_userbase: Any
    install_usersite: Any
    compile: Any
    optimize: Any
    extra_path: Any
    install_path_file: int
    force: int
    skip_build: int
    warn_dir: int
    build_base: Any
    build_lib: Any
    record: Any
    def initialize_options(self) -> None: ...
    config_vars: Any
    install_libbase: Any
    def finalize_options(self) -> None: ...
    def dump_dirs(self, msg) -> None: ...
    def finalize_unix(self) -> None: ...
    def finalize_other(self) -> None: ...
    def select_scheme(self, name) -> None: ...
    def expand_basedirs(self) -> None: ...
    def expand_dirs(self) -> None: ...
    def convert_paths(self, *names) -> None: ...
    path_file: Any
    extra_dirs: Any
    def handle_extra_path(self) -> None: ...
    def change_roots(self, *names) -> None: ...
    def create_home_path(self) -> None: ...
    def run(self) -> None: ...
    def create_path_file(self) -> None: ...
    def get_outputs(self): ...
    def get_inputs(self): ...
    def has_lib(self): ...
    def has_headers(self): ...
    def has_scripts(self): ...
    def has_data(self): ...
    sub_commands: Any



import sys
from collections.abc import Container, Iterable, Sequence
from types import ModuleType
from typing import Any
from typing_extensions import Literal

if sys.platform == "win32":
    from _msi import *
    from _msi import _Database

    AMD64: bool
    Win64: bool

    datasizemask: Literal[0x00FF]
    type_valid: Literal[0x0100]
    type_localizable: Literal[0x0200]
    typemask: Literal[0x0C00]
    type_long: Literal[0x0000]
    type_short: Literal[0x0400]
    type_string: Literal[0x0C00]
    type_binary: Literal[0x0800]
    type_nullable: Literal[0x1000]
    type_key: Literal[0x2000]
    knownbits: Literal[0x3FFF]

    class Table:
        name: str
        fields: list[tuple[int, str, int]]
        def __init__(self, name: str) -> None: ...
        def add_field(self, index: int, name: str, type: int) -> None: ...
        def sql(self) -> str: ...
        def create(self, db: _Database) -> None: ...

    class _Unspecified: ...

    def change_sequence(
        seq: Sequence[tuple[str, str | None, int]],
        action: str,
        seqno: int | type[_Unspecified] = ...,
        cond: str | type[_Unspecified] = ...,
    ) -> None: ...
    def add_data(db: _Database, table: str, values: Iterable[tuple[Any, ...]]) -> None: ...
    def add_stream(db: _Database, name: str, path: str) -> None: ...
    def init_database(
        name: str, schema: ModuleType, ProductName: str, ProductCode: str, ProductVersion: str, Manufacturer: str
    ) -> _Database: ...
    def add_tables(db: _Database, module: ModuleType) -> None: ...
    def make_id(str: str) -> str: ...
    def gen_uuid() -> str: ...

    class CAB:
        name: str
        files: list[tuple[str, str]]
        filenames: set[str]
        index: int
        def __init__(self, name: str) -> None: ...
        def gen_id(self, file: str) -> str: ...
        def append(self, full: str, file: str, logical: str) -> tuple[int, str]: ...
        def commit(self, db: _Database) -> None: ...
    _directories: set[str]

    class Directory:
        db: _Database
        cab: CAB
        basedir: str
        physical: str
        logical: str
        component: str | None
        short_names: set[str]
        ids: set[str]
        keyfiles: dict[str, str]
        componentflags: int | None
        absolute: str
        def __init__(
            self,
            db: _Database,
            cab: CAB,
            basedir: str,
            physical: str,
            _logical: str,
            default: str,
            componentflags: int | None = None,
        ) -> None: ...
        def start_component(
            self,
            component: str | None = None,
            feature: Feature | None = None,
            flags: int | None = None,
            keyfile: str | None = None,
            uuid: str | None = None,
        ) -> None: ...
        def make_short(self, file: str) -> str: ...
        def add_file(self, file: str, src: str | None = None, version: str | None = None, language: str | None = None) -> str: ...
        def glob(self, pattern: str, exclude: Container[str] | None = None) -> list[str]: ...
        def remove_pyc(self) -> None: ...

    class Binary:
        name: str
        def __init__(self, fname: str) -> None: ...

    class Feature:
        id: str
        def __init__(
            self,
            db: _Database,
            id: str,
            title: str,
            desc: str,
            display: int,
            level: int = 1,
            parent: Feature | None = None,
            directory: str | None = None,
            attributes: int = 0,
        ) -> None: ...
        def set_current(self) -> None: ...

    class Control:
        dlg: Dialog
        name: str
        def __init__(self, dlg: Dialog, name: str) -> None: ...
        def event(self, event: str, argument: str, condition: str = "1", ordering: int | None = None) -> None: ...
        def mapping(self, event: str, attribute: str) -> None: ...
        def condition(self, action: str, condition: str) -> None: ...

    class RadioButtonGroup(Control):
        property: str
        index: int
        def __init__(self, dlg: Dialog, name: str, property: str) -> None: ...
        def add(self, name: str, x: int, y: int, w: int, h: int, text: str, value: str | None = None) -> None: ...

    class Dialog:
        db: _Database
        name: str
        x: int
        y: int
        w: int
        h: int
        def __init__(
            self,
            db: _Database,
            name: str,
            x: int,
            y: int,
            w: int,
            h: int,
            attr: int,
            title: str,
            first: str,
            default: str,
            cancel: str,
        ) -> None: ...
        def control(
            self,
            name: str,
            type: str,
            x: int,
            y: int,
            w: int,
            h: int,
            attr: int,
            prop: str | None,
            text: str | None,
            next: str | None,
            help: str | None,
        ) -> Control: ...
        def text(self, name: str, x: int, y: int, w: int, h: int, attr: int, text: str | None) -> Control: ...
        def bitmap(self, name: str, x: int, y: int, w: int, h: int, text: str | None) -> Control: ...
        def line(self, name: str, x: int, y: int, w: int, h: int) -> Control: ...
        def pushbutton(
            self, name: str, x: int, y: int, w: int, h: int, attr: int, text: str | None, next: str | None
        ) -> Control: ...
        def radiogroup(
            self, name: str, x: int, y: int, w: int, h: int, attr: int, prop: str | None, text: str | None, next: str | None
        ) -> RadioButtonGroup: ...
        def checkbox(
            self, name: str, x: int, y: int, w: int, h: int, attr: int, prop: str | None, text: str | None, next: str | None
        ) -> Control: ...



import datetime
from typing import List, Union

import pytz
from django.db.models import Model
from django.db.models.fields.files import ImageFieldFile
from rest_framework.exceptions import ValidationError

from nwon_django_toolbox.typings import AllowedUpdate


def validate_model_update(
    old_model_instance: Model,
    updated_model_instance: Model,
    allowed_updates: Union[List[AllowedUpdate], None],
):
    """
    Checks an update model instance against an allowed set of fields and their values
    """

    if updated_model_instance._state.adding is True:
        return

    if allowed_updates is None:
        return

    fields = updated_model_instance._meta.get_fields()

    allowed_field_names = [update.field_name for update in allowed_updates]

    if old_model_instance.__class__ != updated_model_instance.__class__:
        raise ValidationError(
            f"You tried to validate update for two instances of a different class: "
            f"{old_model_instance.__class__}  and "
            f"{updated_model_instance.__class__}"
        )

    if old_model_instance.pk != updated_model_instance.pk:
        raise ValidationError(
            f"You tried to validate update for two instances of "
            f"{old_model_instance.__class__} with a different pk value: "
            f"{old_model_instance.pk} and {updated_model_instance.pk}"
        )

    for field in fields:
        field_name = field.name
        old_value = getattr(old_model_instance, field_name)
        new_value = getattr(updated_model_instance, field_name)

        if isinstance(old_value, ImageFieldFile) and isinstance(
            new_value, ImageFieldFile
        ):
            old_value = old_value.url if old_value else None
            new_value = new_value.url if new_value else None

        if isinstance(old_value, datetime.datetime) and old_value.tzinfo is None:
            old_value = pytz.utc.localize(old_value)

        if isinstance(new_value, datetime.datetime) and new_value.tzinfo is None:
            new_value = pytz.utc.localize(new_value)

        """ 
        When working with polymorphic models it was problematic to compare the full 
        instances. Therefore we compare the pk values instead.

        If the model type is wrong the model itself will raise an error when saving.
        """
        if isinstance(old_value, Model):
            old_value = old_value.pk

        if isinstance(new_value, Model):
            new_value = new_value.pk

        if old_value == new_value:
            continue

        if field_name not in allowed_field_names:
            raise ValidationError(
                {
                    field_name: f"You are not allowed to update {field_name} "
                    f"with value {new_value} from {old_value} for "
                    f"{updated_model_instance.__class__} "
                    f"with pk {updated_model_instance.pk}"
                }
            )

        allowed_updates_for_field_name = _filter_allowed_updates(
            allowed_updates, field_name
        )

        # There is some configuration which limits possible field values
        new_value_has_constraint = any(
            x.allowed_values is not None for x in allowed_updates_for_field_name
        )

        # The new value is explicitly allowed in some field configuration
        new_value_matches_constraint = any(
            new_value in x.allowed_values
            for x in allowed_updates_for_field_name
            if x.allowed_values
        )

        if new_value and new_value_has_constraint and not new_value_matches_constraint:
            raise ValidationError(
                {
                    field_name: f"{new_value} can't be set as value for "
                    f"{field_name} in {updated_model_instance} with pk "
                    f"{updated_model_instance.pk}"
                }
            )


def _filter_allowed_updates(
    allowed_updates: List[AllowedUpdate], field_name: str
) -> List[AllowedUpdate]:
    return list(
        filter(
            lambda x: field_name == x.field_name and x.allowed_values,
            allowed_updates,
        )
    )



from collections.abc import Mapping
from pathlib import Path
from typing import Any, Protocol

from django.contrib.gis.gdal import DataSource, OGRGeomType
from django.contrib.gis.gdal.field import Field as OGRField
from django.contrib.gis.gdal.layer import Layer
from django.db.backends.base.operations import BaseDatabaseOperations
from django.db.models import Field, Model

class LayerMapError(Exception): ...
class InvalidString(LayerMapError): ...
class InvalidDecimal(LayerMapError): ...
class InvalidInteger(LayerMapError): ...
class MissingForeignKey(LayerMapError): ...

class _Writer(Protocol):
    def write(self, __s: str) -> Any: ...

class LayerMapping:
    MULTI_TYPES: dict[int, OGRGeomType]
    FIELD_TYPES: dict[Field, OGRField | tuple[OGRField, ...]]
    ds: DataSource
    layer: Layer
    using: str
    spatial_backend: BaseDatabaseOperations
    mapping: Mapping[str, str]
    model: type[Model]
    geo_field: Any
    source_srs: Any
    transform: Any
    encoding: str | None
    unique: list[str] | tuple[str, ...] | str | None
    transaction_mode: Any
    transaction_decorator: Any
    def __init__(
        self,
        model: type[Model],
        data: str | Path | DataSource,
        mapping: Mapping[str, str],
        layer: int = ...,
        source_srs: Any | None = ...,
        encoding: str = ...,
        transaction_mode: str = ...,
        transform: bool = ...,
        unique: list[str] | tuple[str, ...] | str | None = ...,
        using: str | None = ...,
    ) -> None: ...
    def check_fid_range(self, fid_range: Any) -> Any: ...
    geom_field: str
    fields: dict[str, Field]
    coord_dim: int
    def check_layer(self) -> Any: ...
    def check_srs(self, source_srs: Any) -> Any: ...
    def check_unique(self, unique: Any) -> None: ...
    def feature_kwargs(self, feat: Any) -> Any: ...
    def unique_kwargs(self, kwargs: Any) -> Any: ...
    def verify_ogr_field(self, ogr_field: Any, model_field: Any) -> Any: ...
    def verify_fk(self, feat: Any, rel_model: Any, rel_mapping: Any) -> Any: ...
    def verify_geom(self, geom: Any, model_field: Any) -> Any: ...
    def coord_transform(self) -> Any: ...
    def geometry_field(self) -> Any: ...
    def make_multi(self, geom_type: Any, model_field: Any) -> Any: ...
    def save(
        self,
        verbose: bool = ...,
        fid_range: bool = ...,
        step: bool = ...,
        progress: bool = ...,
        silent: bool = ...,
        stream: _Writer = ...,
        strict: bool = ...,
    ) -> Any: ...



from nwon_django_toolbox.tests.helper.check_object_against_parameter import (
    check_object_against_parameter,
)
from nwon_django_toolbox.tests.helper.dictionary_is_serialized_instance import (
    dictionary_is_serialized_instance,
)
from nwon_django_toolbox.tests.helper.file_exists_in_storage import (
    file_exists_in_storage,
)
from nwon_django_toolbox.tests.helper.random_image import (
    get_path_to_random_image,
    get_random_image,
)
from nwon_django_toolbox.tests.helper.random_temporary_file import (
    get_random_tempfile_path,
)

__all__ = [
    "file_exists_in_storage",
    "check_object_against_parameter",
    "get_path_to_random_image",
    "get_random_image",
    "get_random_tempfile_path",
    "dictionary_is_serialized_instance",
]



from collections.abc import Sequence
from typing import Any

from django.db.models.sql.where import NothingNode
from typing_extensions import TypeAlias

_NodeChildren: TypeAlias = list[Node | NothingNode | Sequence[Any]]

class Node:
    children: _NodeChildren
    default: str
    connector: str
    negated: bool
    def __init__(
        self, children: _NodeChildren | None = ..., connector: str | None = ..., negated: bool = ...
    ) -> None: ...
    def __deepcopy__(self, memodict: dict[Any, Any]) -> Node: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __contains__(self, other: tuple[str, int]) -> bool: ...
    def __hash__(self) -> int: ...
    def add(self, data: Any, conn_type: str, squash: bool = ...) -> Any: ...
    def negate(self) -> None: ...



from logging import Logger
from typing import Any

from django.forms.widgets import Textarea

geo_context: Any
logger: Logger

class OpenLayersWidget(Textarea):
    def get_context(self, name: Any, value: Any, attrs: Any) -> Any: ...
    def map_options(self) -> Any: ...



from celery import app as app
from celery import local as local
from celery import result as result
from celery import schedules as schedules
from celery import states as states
from celery._state import current_app as current_app
from celery._state import current_task as current_task
from celery.app import Task as Task
from celery.app import shared_task as shared_task
from celery.app import task as task
from celery.app.base import Celery as Celery
from celery.apps import worker as worker
from celery.canvas import Signature as Signature
from celery.canvas import chain as chain
from celery.canvas import chord as chord
from celery.canvas import chunks as chunks
from celery.canvas import group as group
from celery.canvas import signature as signature
from celery.canvas import xmap as xmap
from celery.canvas import xstarmap as xstarmap
from celery.utils import uuid as uuid

__all__ = [
    "states",
    "result",
    "app",
    "schedules",
    "shared_task",
    "current_app",
    "Celery",
]



from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper

TEST_DATABASE_PREFIX: str

class BaseDatabaseCreation:
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def create_test_db(
        self, verbosity: int = ..., autoclobber: bool = ..., serialize: bool = ..., keepdb: bool = ...
    ) -> str: ...
    def set_as_test_mirror(self, primary_settings_dict: dict[str, dict[str, None] | int | str | None]) -> None: ...
    def serialize_db_to_string(self) -> str: ...
    def deserialize_db_from_string(self, data: str) -> None: ...
    def clone_test_db(self, suffix: Any, verbosity: int = ..., autoclobber: bool = ..., keepdb: bool = ...) -> None: ...
    def get_test_db_clone_settings(self, suffix: str) -> dict[str, Any]: ...
    def destroy_test_db(
        self,
        old_database_name: str | None = ...,
        verbosity: int = ...,
        keepdb: bool = ...,
        suffix: str | None = ...,
    ) -> None: ...
    def sql_table_creation_suffix(self) -> str: ...
    def test_db_signature(self) -> tuple[str, str, str, str]: ...



import importlib.abc
import importlib.machinery
import sys
import types
from _typeshed import ReadableBuffer, StrOrBytesPath
from collections.abc import Callable
from typing import Any
from typing_extensions import ParamSpec

_P = ParamSpec("_P")

if sys.version_info < (3, 12):
    def module_for_loader(fxn: Callable[_P, types.ModuleType]) -> Callable[_P, types.ModuleType]: ...
    def set_loader(fxn: Callable[_P, types.ModuleType]) -> Callable[_P, types.ModuleType]: ...
    def set_package(fxn: Callable[_P, types.ModuleType]) -> Callable[_P, types.ModuleType]: ...

def resolve_name(name: str, package: str | None) -> str: ...

MAGIC_NUMBER: bytes

def cache_from_source(path: str, debug_override: bool | None = None, *, optimization: Any | None = None) -> str: ...
def source_from_cache(path: str) -> str: ...
def decode_source(source_bytes: ReadableBuffer) -> str: ...
def find_spec(name: str, package: str | None = None) -> importlib.machinery.ModuleSpec | None: ...
def spec_from_loader(
    name: str, loader: importlib.abc.Loader | None, *, origin: str | None = None, is_package: bool | None = None
) -> importlib.machinery.ModuleSpec | None: ...
def spec_from_file_location(
    name: str,
    location: StrOrBytesPath | None = None,
    *,
    loader: importlib.abc.Loader | None = None,
    submodule_search_locations: list[str] | None = ...,
) -> importlib.machinery.ModuleSpec | None: ...
def module_from_spec(spec: importlib.machinery.ModuleSpec) -> types.ModuleType: ...

class LazyLoader(importlib.abc.Loader):
    def __init__(self, loader: importlib.abc.Loader) -> None: ...
    @classmethod
    def factory(cls, loader: importlib.abc.Loader) -> Callable[..., LazyLoader]: ...
    def exec_module(self, module: types.ModuleType) -> None: ...

def source_hash(source_bytes: ReadableBuffer) -> bytes: ...



from collections.abc import Generator
from contextlib import contextmanager
from typing import Any

from django.test import LiveServerTestCase

class SeleniumTestCaseBase:
    browsers: Any
    browser: Any
    @classmethod
    def import_webdriver(cls, browser: Any) -> type[Any]: ...  # Type[WebDriver]
    def create_webdriver(self) -> Any: ...  # WebDriver

class SeleniumTestCase(LiveServerTestCase):
    implicit_wait: int
    selenium: Any
    @contextmanager
    def disable_implicit_wait(self) -> Generator[None, None, None]: ...



from collections.abc import Iterable
from typing import Any

from django.db.backends.base.client import BaseDatabaseClient
from django.db.backends.oracle.base import DatabaseWrapper

class DatabaseClient(BaseDatabaseClient):
    connection: DatabaseWrapper
    executable_name: str
    wrapper_name: str
    @staticmethod
    def connect_string(settings_dict: dict[str, Any]) -> str: ...
    @classmethod
    def settings_to_cmd_args_env(
        cls, settings_dict: dict[str, Any], parameters: Iterable[str]
    ) -> tuple[list[str], None]: ...



from collections.abc import Callable, Sequence
from typing import Any

from django import http
from django.http.response import HttpResponseBase, HttpResponseRedirect
from django.utils.functional import _StrOrPromise

class AccessMixin:
    login_url: Any
    permission_denied_message: _StrOrPromise
    raise_exception: bool
    redirect_field_name: Any
    def get_login_url(self) -> str: ...
    def get_permission_denied_message(self) -> _StrOrPromise: ...
    def get_redirect_field_name(self) -> str: ...
    def handle_no_permission(self) -> HttpResponseRedirect: ...

class LoginRequiredMixin(AccessMixin):
    def dispatch(self, request: http.HttpRequest, *args: Any, **kwargs: Any) -> HttpResponseBase: ...

class PermissionRequiredMixin(AccessMixin):
    permission_required: Any
    def get_permission_required(self) -> Sequence[str]: ...
    def has_permission(self) -> bool: ...
    def dispatch(self, request: http.HttpRequest, *args: Any, **kwargs: Any) -> HttpResponseBase: ...

class UserPassesTestMixin(AccessMixin):
    def test_func(self) -> bool | None: ...
    def get_test_func(self) -> Callable: ...
    def dispatch(self, request: http.HttpRequest, *args: Any, **kwargs: Any) -> HttpResponseBase: ...



from collections.abc import Sequence
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.base import Model
from django.db.utils import DatabaseError

class BaseDatabaseFeatures:
    minimum_database_version: tuple[int, ...] | None
    gis_enabled: bool
    allows_group_by_lob: bool
    allows_group_by_pk: bool
    allows_group_by_selected_pks: bool
    empty_fetchmany_value: Sequence[Any]
    update_can_self_select: bool
    interprets_empty_strings_as_nulls: bool
    supports_nullable_unique_constraints: bool
    supports_partially_nullable_unique_constraints: bool
    supports_deferrable_unique_constraints: bool
    can_use_chunked_reads: bool
    can_return_columns_from_insert: bool
    can_return_rows_from_bulk_insert: bool
    has_bulk_insert: bool
    uses_savepoints: bool
    can_release_savepoints: bool
    related_fields_match_type: bool
    allow_sliced_subqueries_with_in: bool
    has_select_for_update: bool
    has_select_for_update_nowait: bool
    has_select_for_update_skip_locked: bool
    has_select_for_update_of: bool
    has_select_for_no_key_update: bool
    select_for_update_of_column: bool
    test_db_allows_multiple_connections: bool
    supports_unspecified_pk: bool
    supports_forward_references: bool
    truncates_names: bool
    has_real_datatype: bool
    supports_subqueries_in_group_by: bool
    has_native_uuid_field: bool
    has_native_duration_field: bool
    supports_temporal_subtraction: bool
    supports_regex_backreferencing: bool
    supports_date_lookup_using_string: bool
    supports_timezones: bool
    has_zoneinfo_database: bool
    requires_explicit_null_ordering_when_grouping: bool
    nulls_order_largest: bool
    supports_order_by_nulls_modifier: bool
    order_by_nulls_first: bool
    max_query_params: int | None
    allows_auto_pk_0: bool
    can_defer_constraint_checks: bool
    supports_tablespaces: bool
    supports_sequence_reset: bool
    can_introspect_default: bool
    can_introspect_foreign_keys: bool
    introspected_field_types: dict[str, str]
    supports_index_column_ordering: bool
    can_introspect_materialized_views: bool
    can_distinct_on_fields: bool
    atomic_transactions: bool
    can_rollback_ddl: bool
    supports_atomic_references_rename: bool
    supports_combined_alters: bool
    supports_foreign_keys: bool
    can_create_inline_fk: bool
    can_rename_index: bool
    indexes_foreign_keys: bool
    supports_column_check_constraints: bool
    supports_table_check_constraints: bool
    can_introspect_check_constraints: bool
    supports_paramstyle_pyformat: bool
    requires_literal_defaults: bool
    connection_persists_old_columns: bool
    closed_cursor_error_class: type[DatabaseError]
    has_case_insensitive_like: bool
    bare_select_suffix: str
    implied_column_null: bool
    supports_select_for_update_with_limit: bool
    greatest_least_ignores_nulls: bool
    can_clone_databases: bool
    ignores_table_name_case: bool
    for_update_after_from: bool
    supports_select_union: bool
    supports_select_intersection: bool
    supports_select_difference: bool
    supports_slicing_ordering_in_compound: bool
    supports_parentheses_in_compound: bool
    supports_aggregate_filter_clause: bool
    supports_index_on_text_field: bool
    supports_over_clause: bool
    supports_frame_range_fixed_distance: bool
    only_supports_unbounded_with_preceding_and_following: bool
    supports_cast_with_precision: bool
    time_cast_precision: int
    create_test_procedure_without_params_sql: str | None
    create_test_procedure_with_int_param_sql: str | None
    supports_callproc_kwargs: bool
    supported_explain_formats: set[str]
    supports_default_in_lead_lag: bool
    supports_ignore_conflicts: bool
    supports_update_conflicts: bool
    supports_update_conflicts_with_target: bool
    requires_casted_case_in_updates: bool
    supports_partial_indexes: bool
    supports_functions_in_partial_indexes: bool
    supports_covering_indexes: bool
    supports_expression_indexes: bool
    collate_as_index_expression: bool
    allows_multiple_constraints_on_same_fields: bool
    supports_boolean_expr_in_select_clause: bool
    supports_json_field: bool
    can_introspect_json_field: bool
    supports_primitives_in_json_field: bool
    has_native_json_field: bool
    has_json_operators: bool
    supports_json_field_contains: bool
    json_key_contains_list_matching_requires_list: bool
    has_json_object_function: bool
    supports_collation_on_charfield: bool
    supports_collation_on_textfield: bool
    supports_non_deterministic_collations: bool
    test_collations: dict[str, str | None]
    test_now_utc_template: str | None
    django_test_expected_failures: set[str]
    django_test_skips: dict[str, set[str]]
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    @property
    def supports_explaining_query_execution(self) -> bool: ...
    @property
    def supports_transactions(self) -> bool: ...
    def allows_group_by_selected_pks_on_model(self, model: type[Model]) -> bool: ...



from collections.abc import Callable, Collection, Container, Iterator, Mapping, Sequence
from typing import (  # noqa: Y037  # https://github.com/python/mypy/issues/12211
    Any,
    ClassVar,
    Generic,
    TypeVar,
    Union,
    overload,
)
from uuid import UUID

from django.db import models
from django.db.models import ForeignKey
from django.db.models.base import Model
from django.db.models.fields import _AllLimitChoicesTo, _ChoicesCallable, _FieldChoices, _LimitChoicesTo
from django.db.models.manager import Manager
from django.db.models.query import QuerySet
from django.forms.fields import CallableChoiceIterator, ChoiceField, Field, _ClassLevelWidgetT
from django.forms.forms import BaseForm, DeclarativeFieldsMetaclass
from django.forms.formsets import BaseFormSet
from django.forms.renderers import BaseRenderer
from django.forms.utils import ErrorList, _DataT, _FilesT
from django.forms.widgets import Widget
from django.utils.datastructures import _IndexableCollection, _ListOrTuple, _PropertyDescriptor
from django.utils.functional import _StrOrPromise
from typing_extensions import Literal, TypeAlias

ALL_FIELDS: Literal["__all__"]

# https://github.com/python/mypy/issues/12211
_Fields: TypeAlias = Union[_ListOrTuple[str], Literal["__all__"]]
_Widgets: TypeAlias = dict[str, type[Widget] | Widget]

_Labels: TypeAlias = dict[str, str]
_HelpTexts: TypeAlias = dict[str, str]
_ErrorMessages: TypeAlias = dict[str, dict[str, str]]
_FormFieldCallback: TypeAlias = Callable[[models.Field], Field]

_M = TypeVar("_M", bound=Model)
_ParentM = TypeVar("_ParentM", bound=Model)

def construct_instance(
    form: BaseForm, instance: _M, fields: Container[str] | None = ..., exclude: Container[str] | None = ...
) -> _M: ...
def model_to_dict(instance: Model, fields: _Fields | None = ..., exclude: _Fields | None = ...) -> dict[str, Any]: ...
def apply_limit_choices_to_to_formfield(formfield: Field) -> None: ...
def fields_for_model(
    model: type[Model],
    fields: _Fields | None = ...,
    exclude: _Fields | None = ...,
    widgets: _Widgets | None = ...,
    formfield_callback: _FormFieldCallback | None = ...,
    localized_fields: _Fields | None = ...,
    labels: _Labels | None = ...,
    help_texts: _HelpTexts | None = ...,
    error_messages: _ErrorMessages | None = ...,
    field_classes: Mapping[str, type[Field]] | None = ...,
    *,
    apply_limit_choices_to: bool = ...,
) -> dict[str, Any]: ...

class ModelFormOptions(Generic[_M]):
    model: type[_M]
    fields: _Fields | None
    exclude: _Fields | None
    widgets: _Widgets | None
    localized_fields: _Fields | None
    labels: _Labels | None
    help_texts: _HelpTexts | None
    error_messages: _ErrorMessages | None
    field_classes: dict[str, type[Field]] | None
    def __init__(self, options: type | None = ...) -> None: ...

class ModelFormMetaclass(DeclarativeFieldsMetaclass): ...

class BaseModelForm(Generic[_M], BaseForm):
    instance: _M
    _meta: ModelFormOptions[_M]
    def __init__(
        self,
        data: _DataT | None = ...,
        files: _FilesT | None = ...,
        auto_id: bool | str = ...,
        prefix: str | None = ...,
        initial: Mapping[str, Any] | None = ...,
        error_class: type[ErrorList] = ...,
        label_suffix: str | None = ...,
        empty_permitted: bool = ...,
        instance: _M | None = ...,
        use_required_attribute: bool | None = ...,
        renderer: BaseRenderer = ...,
    ) -> None: ...
    def validate_unique(self) -> None: ...
    def save(self, commit: bool = ...) -> _M: ...
    def save_m2m(self) -> None: ...

class ModelForm(BaseModelForm[_M], metaclass=ModelFormMetaclass):
    base_fields: ClassVar[dict[str, Field]]

def modelform_factory(
    model: type[_M],
    form: type[ModelForm[_M]] = ...,
    fields: _Fields | None = ...,
    exclude: _Fields | None = ...,
    formfield_callback: _FormFieldCallback | None = ...,
    widgets: _Widgets | None = ...,
    localized_fields: _Fields | None = ...,
    labels: _Labels | None = ...,
    help_texts: _HelpTexts | None = ...,
    error_messages: _ErrorMessages | None = ...,
    field_classes: Mapping[str, type[Field]] | None = ...,
) -> type[ModelForm[_M]]: ...

_ModelFormT = TypeVar("_ModelFormT", bound=ModelForm)

class BaseModelFormSet(Generic[_M, _ModelFormT], BaseFormSet[_ModelFormT]):
    model: type[_M]
    unique_fields: Collection[str]
    queryset: QuerySet[_M] | None
    initial_extra: Sequence[dict[str, Any]] | None
    def __init__(
        self,
        data: _DataT | None = ...,
        files: _FilesT | None = ...,
        auto_id: str = ...,
        prefix: str | None = ...,
        queryset: QuerySet[_M] | None = ...,
        *,
        initial: Sequence[dict[str, Any]] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def initial_form_count(self) -> int: ...
    def get_queryset(self) -> _IndexableCollection[_M]: ...
    def save_new(self, form: _ModelFormT, commit: bool = ...) -> _M: ...
    def save_existing(self, form: _ModelFormT, instance: _M, commit: bool = ...) -> _M: ...
    def delete_existing(self, obj: _M, commit: bool = ...) -> None: ...
    saved_forms: list[_ModelFormT]
    def save_m2m(self) -> None: ...
    def save(self, commit: bool = ...) -> list[_M]: ...
    def clean(self) -> None: ...
    def validate_unique(self) -> None: ...
    def get_unique_error_message(self, unique_check: Sequence[str]) -> str: ...
    def get_date_error_message(self, date_check: tuple[str, Literal["date", "year", "month"], str, str]) -> str: ...
    def get_form_error(self) -> str: ...
    changed_objects: list[tuple[_M, list[str]]]
    deleted_objects: list[_M]
    def save_existing_objects(self, commit: bool = ...) -> list[_M]: ...
    new_objects: list[_M]
    def save_new_objects(self, commit: bool = ...) -> list[_M]: ...
    def add_fields(self, form: _ModelFormT, index: int | None) -> None: ...

def modelformset_factory(
    model: type[_M],
    form: type[_ModelFormT] = ...,
    formfield_callback: _FormFieldCallback | None = ...,
    formset: type[BaseModelFormSet] = ...,
    extra: int = ...,
    can_delete: bool = ...,
    can_order: bool = ...,
    max_num: int | None = ...,
    fields: _Fields | None = ...,
    exclude: _Fields | None = ...,
    widgets: _Widgets | None = ...,
    validate_max: bool = ...,
    localized_fields: _Fields | None = ...,
    labels: _Labels | None = ...,
    help_texts: _HelpTexts | None = ...,
    error_messages: _ErrorMessages | None = ...,
    min_num: int | None = ...,
    validate_min: bool = ...,
    field_classes: Mapping[str, type[Field]] | None = ...,
    absolute_max: int | None = ...,
    can_delete_extra: bool = ...,
) -> type[BaseModelFormSet[_M, _ModelFormT]]: ...

class BaseInlineFormSet(Generic[_M, _ParentM, _ModelFormT], BaseModelFormSet[_M, _ModelFormT]):
    instance: _ParentM
    save_as_new: bool
    unique_fields: Collection[str]
    fk: ForeignKey  # set by inlineformset_set
    def __init__(
        self,
        data: _DataT | None = ...,
        files: _FilesT | None = ...,
        instance: _ParentM | None = ...,
        save_as_new: bool = ...,
        prefix: str | None = ...,
        queryset: QuerySet[_M] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def initial_form_count(self) -> int: ...
    @classmethod
    def get_default_prefix(cls) -> str: ...
    def save_new(self, form: _ModelFormT, commit: bool = ...) -> _M: ...
    def add_fields(self, form: _ModelFormT, index: int | None) -> None: ...
    def get_unique_error_message(self, unique_check: Sequence[str]) -> str: ...

def inlineformset_factory(
    parent_model: type[_ParentM],
    model: type[_M],
    form: type[_ModelFormT] = ...,
    formset: type[BaseInlineFormSet] = ...,
    fk_name: str | None = ...,
    fields: _Fields | None = ...,
    exclude: _Fields | None = ...,
    extra: int = ...,
    can_order: bool = ...,
    can_delete: bool = ...,
    max_num: int | None = ...,
    formfield_callback: _FormFieldCallback | None = ...,
    widgets: _Widgets | None = ...,
    validate_max: bool = ...,
    localized_fields: Sequence[str] | None = ...,
    labels: _Labels | None = ...,
    help_texts: _HelpTexts | None = ...,
    error_messages: _ErrorMessages | None = ...,
    min_num: int | None = ...,
    validate_min: bool = ...,
    field_classes: Mapping[str, type[Field]] | None = ...,
    absolute_max: int | None = ...,
    can_delete_extra: bool = ...,
) -> type[BaseInlineFormSet[_M, _ParentM, _ModelFormT]]: ...

class InlineForeignKeyField(Field):
    disabled: bool
    help_text: _StrOrPromise
    required: bool
    show_hidden_initial: bool
    widget: _ClassLevelWidgetT
    default_error_messages: dict[str, str]
    parent_instance: Model
    pk_field: bool
    to_field: str | None
    def __init__(
        self,
        parent_instance: Model,
        *args: Any,
        pk_field: bool = ...,
        to_field: str | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def clean(self, value: Any) -> Model: ...
    def has_changed(self, initial: Any, data: Any) -> bool: ...

class ModelChoiceIteratorValue:
    def __init__(self, value: Any, instance: Model) -> None: ...

class ModelChoiceIterator:
    field: ModelChoiceField
    queryset: QuerySet
    def __init__(self, field: ModelChoiceField) -> None: ...
    def __iter__(self) -> Iterator[tuple[ModelChoiceIteratorValue | str, str]]: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def choice(self, obj: Model) -> tuple[ModelChoiceIteratorValue, str]: ...

class ModelChoiceField(ChoiceField):
    disabled: bool
    error_messages: dict[str, str]
    help_text: _StrOrPromise
    required: bool
    show_hidden_initial: bool
    validators: list[Any]
    default_error_messages: dict[str, str]
    iterator: type[ModelChoiceIterator]
    empty_label: _StrOrPromise | None
    queryset: QuerySet[models.Model] | None
    limit_choices_to: _AllLimitChoicesTo | None
    to_field_name: str | None
    def __init__(
        self,
        queryset: None | Manager[models.Model] | QuerySet[models.Model],
        *,
        empty_label: _StrOrPromise | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        to_field_name: str | None = ...,
        limit_choices_to: _AllLimitChoicesTo | None = ...,
        blank: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    def get_limit_choices_to(self) -> _LimitChoicesTo: ...
    def label_from_instance(self, obj: Model) -> str: ...
    choices: _PropertyDescriptor[
        _FieldChoices | _ChoicesCallable | CallableChoiceIterator,
        _FieldChoices | CallableChoiceIterator | ModelChoiceIterator,
    ]
    def prepare_value(self, value: Any) -> Any: ...
    def to_python(self, value: Any | None) -> Model | None: ...
    def validate(self, value: Model | None) -> None: ...
    def has_changed(self, initial: Model | int | str | UUID | None, data: int | str | None) -> bool: ...

class ModelMultipleChoiceField(ModelChoiceField):
    disabled: bool
    empty_label: _StrOrPromise | None
    help_text: _StrOrPromise
    required: bool
    show_hidden_initial: bool
    widget: _ClassLevelWidgetT
    hidden_widget: type[Widget]
    default_error_messages: dict[str, str]
    def __init__(self, queryset: None | Manager[Model] | QuerySet[Model], **kwargs: Any) -> None: ...
    def to_python(self, value: Any) -> list[Model]: ...  # type: ignore[override]
    def clean(self, value: Any) -> QuerySet[Model]: ...
    def prepare_value(self, value: Any) -> Any: ...
    def has_changed(self, initial: Collection[Any] | None, data: Collection[Any] | None) -> bool: ...  # type: ignore

def modelform_defines_fields(form_class: type[ModelForm]) -> bool: ...
@overload
def _get_foreign_key(  # type: ignore
    parent_model: type[Model], model: type[Model], fk_name: str | None = ..., can_fail: Literal[True] = ...
) -> ForeignKey | None: ...
@overload
def _get_foreign_key(
    parent_model: type[Model], model: type[Model], fk_name: str | None = ..., can_fail: Literal[False] = ...
) -> ForeignKey: ...
